@@ -9,7 +9,6 @@ sed -i -e 's/http:\/\/deb/https:\/\/deb/g' /etc/apt/sources.list
 apt-get update
 
 apt-get install -y --no-remove \
-        abootimg \
         cpio \
         fastboot \
         netcat \
@@ -19,7 +18,8 @@ apt-get install -y --no-remove \
         python3-serial \
         rsync \
         snmp \
-        wget
+        wget \
+        zstd
 
 # setup SNMPv2 SMI MIB
 wget https://raw.githubusercontent.com/net-snmp/net-snmp/master/mibs/SNMPv2-SMI.txt \
@@ -31,3 +31,15 @@ arch=armhf . .gitlab-ci/container/baremetal_build.sh
 # This firmware file from Debian bullseye causes hangs
 wget https://git.kernel.org/pub/scm/linux/kernel/git/firmware/linux-firmware.git/plain/qcom/a530_pfp.fw?id=d5f9eea5a251d43412b07f5295d03e97b89ac4a5 \
      -O /rootfs-arm64/lib/firmware/qcom/a530_pfp.fw
+
+mkdir -p /baremetal-files/jetson-nano/boot/
+ln -s \
+    /baremetal-files/Image \
+    /baremetal-files/tegra210-p3450-0000.dtb \
+    /baremetal-files/jetson-nano/boot/
+
+mkdir -p /baremetal-files/jetson-tk1/boot/
+ln -s \
+    /baremetal-files/zImage \
+    /baremetal-files/tegra124-jetson-tk1.dtb \
+    /baremetal-files/jetson-tk1/boot/
