@@ -1,5 +1,5 @@
 /*
- * Copyright © 2019 Raspberry Pi
+ * Copyright © 2019 Raspberry Pi Ltd
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -23,11 +23,9 @@
 
 #include "v3dv_private.h"
 
-#include "broadcom/cle/v3dx_pack.h"
 #include "drm-uapi/drm_fourcc.h"
 #include "util/format/u_format.h"
 #include "util/u_math.h"
-#include "vk_format_info.h"
 #include "vk_util.h"
 #include "vulkan/wsi/wsi_common.h"
 
@@ -77,9 +75,9 @@ v3d_setup_slices(struct v3dv_image *image)
 {
    assert(image->cpp > 0);
 
-   uint32_t width = image->extent.width;
-   uint32_t height = image->extent.height;
-   uint32_t depth = image->extent.depth;
+   uint32_t width = image->vk.extent.width;
+   uint32_t height = image->vk.extent.height;
+   uint32_t depth = image->vk.extent.depth;
 
    /* Note that power-of-two padding is based on level 1.  These are not
     * equivalent to just util_next_power_of_two(dimension), because at a
@@ -95,21 +93,21 @@ v3d_setup_slices(struct v3dv_image *image)
    uint32_t uif_block_w = utile_w * 2;
    uint32_t uif_block_h = utile_h * 2;
 
-   uint32_t block_width = vk_format_get_blockwidth(image->vk_format);
-   uint32_t block_height = vk_format_get_blockheight(image->vk_format);
+   uint32_t block_width = vk_format_get_blockwidth(image->vk.format);
+   uint32_t block_height = vk_format_get_blockheight(image->vk.format);
 
-   assert(image->samples == VK_SAMPLE_COUNT_1_BIT ||
-          image->samples == VK_SAMPLE_COUNT_4_BIT);
-   bool msaa = image->samples != VK_SAMPLE_COUNT_1_BIT;
+   assert(image->vk.samples == VK_SAMPLE_COUNT_1_BIT ||
+          image->vk.samples == VK_SAMPLE_COUNT_4_BIT);
+   bool msaa = image->vk.samples != VK_SAMPLE_COUNT_1_BIT;
 
    bool uif_top = msaa;
 
-   assert(image->array_size > 0);
+   assert(image->vk.array_layers > 0);
    assert(depth > 0);
-   assert(image->levels >= 1);
+   assert(image->vk.mip_levels >= 1);
 
    uint32_t offset = 0;
-   for (int32_t i = image->levels - 1; i >= 0; i--) {
+   for (int32_t i = image->vk.mip_levels - 1; i >= 0; i--) {
       struct v3d_resource_slice *slice = &image->slices[i];
 
       uint32_t level_width, level_height, level_depth;
@@ -135,21 +133,21 @@ v3d_setup_slices(struct v3dv_image *image)
       level_height = DIV_ROUND_UP(level_height, block_height);
 
       if (!image->tiled) {
-         slice->tiling = VC5_TILING_RASTER;
-         if (image->type == VK_IMAGE_TYPE_1D)
+         slice->tiling = V3D_TILING_RASTER;
+         if (image->vk.image_type == VK_IMAGE_TYPE_1D)
             level_width = align(level_width, 64 / image->cpp);
       } else {
          if ((i != 0 || !uif_top) &&
              (level_width <= utile_w || level_height <= utile_h)) {
-            slice->tiling = VC5_TILING_LINEARTILE;
+            slice->tiling = V3D_TILING_LINEARTILE;
             level_width = align(level_width, utile_w);
             level_height = align(level_height, utile_h);
          } else if ((i != 0 || !uif_top) && level_width <= uif_block_w) {
-            slice->tiling = VC5_TILING_UBLINEAR_1_COLUMN;
+            slice->tiling = V3D_TILING_UBLINEAR_1_COLUMN;
             level_width = align(level_width, uif_block_w);
             level_height = align(level_height, uif_block_h);
          } else if ((i != 0 || !uif_top) && level_width <= 2 * uif_block_w) {
-            slice->tiling = VC5_TILING_UBLINEAR_2_COLUMN;
+            slice->tiling = V3D_TILING_UBLINEAR_2_COLUMN;
             level_width = align(level_width, 2 * uif_block_w);
             level_height = align(level_height, uif_block_h);
          } else {
@@ -167,10 +165,10 @@ v3d_setup_slices(struct v3dv_image *image)
              * perfectly misaligned.
              */
             if ((level_height / uif_block_h) %
-                (VC5_PAGE_CACHE_SIZE / VC5_UIFBLOCK_ROW_SIZE) == 0) {
-               slice->tiling = VC5_TILING_UIF_XOR;
+                (V3D_PAGE_CACHE_SIZE / V3D_UIFBLOCK_ROW_SIZE) == 0) {
+               slice->tiling = V3D_TILING_UIF_XOR;
             } else {
-               slice->tiling = VC5_TILING_UIF_NO_XOR;
+               slice->tiling = V3D_TILING_UIF_NO_XOR;
             }
          }
       }
@@ -178,8 +176,8 @@ v3d_setup_slices(struct v3dv_image *image)
       slice->offset = offset;
       slice->stride = level_width * image->cpp;
       slice->padded_height = level_height;
-      if (slice->tiling == VC5_TILING_UIF_NO_XOR ||
-          slice->tiling == VC5_TILING_UIF_XOR) {
+      if (slice->tiling == V3D_TILING_UIF_NO_XOR ||
+          slice->tiling == V3D_TILING_UIF_XOR) {
          slice->padded_height_of_output_image_in_uif_blocks =
             slice->padded_height / (2 * v3d_utile_height(image->cpp));
       }
@@ -195,7 +193,7 @@ v3d_setup_slices(struct v3dv_image *image)
       if (i == 1 &&
           level_width > 4 * uif_block_w &&
           level_height > PAGE_CACHE_MINUS_1_5_UB_ROWS * uif_block_h) {
-         slice_total_size = align(slice_total_size, VC5_UIFCFG_PAGE_SIZE);
+         slice_total_size = align(slice_total_size, V3D_UIFCFG_PAGE_SIZE);
       }
 
       offset += slice_total_size;
@@ -211,13 +209,12 @@ v3d_setup_slices(struct v3dv_image *image)
     *
     * We additionally align to 4k, which improves UIF XOR performance.
     */
-   image->alignment =
-      image->tiling == VK_IMAGE_TILING_LINEAR ? image->cpp : 4096;
+   image->alignment = image->tiled ? 4096 : image->cpp;
    uint32_t align_offset =
       align(image->slices[0].offset, image->alignment) - image->slices[0].offset;
    if (align_offset) {
       image->size += align_offset;
-      for (int i = 0; i < image->levels; i++)
+      for (int i = 0; i < image->vk.mip_levels; i++)
          image->slices[i].offset += align_offset;
    }
 
@@ -225,10 +222,10 @@ v3d_setup_slices(struct v3dv_image *image)
     * one full mipmap tree to the next (64b aligned).  For 3D textures,
     * we need to program the stride between slices of miplevel 0.
     */
-   if (image->type != VK_IMAGE_TYPE_3D) {
+   if (image->vk.image_type != VK_IMAGE_TYPE_3D) {
       image->cube_map_stride =
          align(image->slices[0].offset + image->slices[0].size, 64);
-      image->size += image->cube_map_stride * (image->array_size - 1);
+      image->size += image->cube_map_stride * (image->vk.array_layers - 1);
    } else {
       image->cube_map_stride = image->slices[0].size;
    }
@@ -239,110 +236,214 @@ v3dv_layer_offset(const struct v3dv_image *image, uint32_t level, uint32_t layer
 {
    const struct v3d_resource_slice *slice = &image->slices[level];
 
-   if (image->type == VK_IMAGE_TYPE_3D)
+   if (image->vk.image_type == VK_IMAGE_TYPE_3D)
       return image->mem_offset + slice->offset + layer * slice->size;
    else
       return image->mem_offset + slice->offset + layer * image->cube_map_stride;
 }
 
 VkResult
-v3dv_CreateImage(VkDevice _device,
-                 const VkImageCreateInfo *pCreateInfo,
-                 const VkAllocationCallbacks *pAllocator,
-                 VkImage *pImage)
+v3dv_image_init(struct v3dv_device *device,
+                const VkImageCreateInfo *pCreateInfo,
+                const VkAllocationCallbacks *pAllocator,
+                struct v3dv_image *image)
 {
-   V3DV_FROM_HANDLE(v3dv_device, device, _device);
-   struct v3dv_image *image = NULL;
-
-   assert(pCreateInfo->sType == VK_STRUCTURE_TYPE_IMAGE_CREATE_INFO);
-
-   v3dv_assert(pCreateInfo->mipLevels > 0);
-   v3dv_assert(pCreateInfo->arrayLayers > 0);
-   v3dv_assert(pCreateInfo->samples > 0);
-   v3dv_assert(pCreateInfo->extent.width > 0);
-   v3dv_assert(pCreateInfo->extent.height > 0);
-   v3dv_assert(pCreateInfo->extent.depth > 0);
-
    /* When using the simulator the WSI common code will see that our
     * driver wsi device doesn't match the display device and because of that
     * it will not attempt to present directly from the swapchain images,
-    * instead it will use the prime blit path (use_prime_blit flag in
+    * instead it will use the prime blit path (use_buffer_blit flag in
     * struct wsi_swapchain), where it copies the contents of the swapchain
     * images to a linear buffer with appropriate row stride for presentation.
     * As a result, on that path, swapchain images do not have any special
     * requirements and are not created with the pNext structs below.
     */
+   VkImageTiling tiling = pCreateInfo->tiling;
    uint64_t modifier = DRM_FORMAT_MOD_INVALID;
-   if (pCreateInfo->tiling == VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT) {
+   if (tiling == VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT) {
       const VkImageDrmFormatModifierListCreateInfoEXT *mod_info =
          vk_find_struct_const(pCreateInfo->pNext,
                               IMAGE_DRM_FORMAT_MODIFIER_LIST_CREATE_INFO_EXT);
-      assert(mod_info);
-      for (uint32_t i = 0; i < mod_info->drmFormatModifierCount; i++) {
-         switch (mod_info->pDrmFormatModifiers[i]) {
-         case DRM_FORMAT_MOD_LINEAR:
-            if (modifier == DRM_FORMAT_MOD_INVALID)
-               modifier = DRM_FORMAT_MOD_LINEAR;
-            break;
-         case DRM_FORMAT_MOD_BROADCOM_UIF:
-            modifier = DRM_FORMAT_MOD_BROADCOM_UIF;
-            break;
+      const VkImageDrmFormatModifierExplicitCreateInfoEXT *explicit_mod_info =
+         vk_find_struct_const(pCreateInfo->pNext,
+                              IMAGE_DRM_FORMAT_MODIFIER_EXPLICIT_CREATE_INFO_EXT);
+      assert(mod_info || explicit_mod_info);
+
+      if (mod_info) {
+         for (uint32_t i = 0; i < mod_info->drmFormatModifierCount; i++) {
+            switch (mod_info->pDrmFormatModifiers[i]) {
+            case DRM_FORMAT_MOD_LINEAR:
+               if (modifier == DRM_FORMAT_MOD_INVALID)
+                  modifier = DRM_FORMAT_MOD_LINEAR;
+               break;
+            case DRM_FORMAT_MOD_BROADCOM_UIF:
+               modifier = DRM_FORMAT_MOD_BROADCOM_UIF;
+               break;
+            }
          }
+      } else {
+         modifier = explicit_mod_info->drmFormatModifier;
       }
-   } else {
-      const struct wsi_image_create_info *wsi_info =
-         vk_find_struct_const(pCreateInfo->pNext, WSI_IMAGE_CREATE_INFO_MESA);
-      if (wsi_info)
-         modifier = DRM_FORMAT_MOD_LINEAR;
+      assert(modifier == DRM_FORMAT_MOD_LINEAR ||
+             modifier == DRM_FORMAT_MOD_BROADCOM_UIF);
+   } else if (pCreateInfo->imageType == VK_IMAGE_TYPE_1D ||
+              image->vk.wsi_legacy_scanout) {
+      tiling = VK_IMAGE_TILING_LINEAR;
    }
 
-   /* 1D and 1D_ARRAY textures are always raster-order */
-   VkImageTiling tiling;
-   if (pCreateInfo->imageType == VK_IMAGE_TYPE_1D)
-      tiling = VK_IMAGE_TILING_LINEAR;
-   else if (modifier == DRM_FORMAT_MOD_INVALID)
-      tiling = pCreateInfo->tiling;
-   else if (modifier == DRM_FORMAT_MOD_BROADCOM_UIF)
-      tiling = VK_IMAGE_TILING_OPTIMAL;
-   else
-      tiling = VK_IMAGE_TILING_LINEAR;
+#ifdef ANDROID
+   const VkNativeBufferANDROID *native_buffer =
+      vk_find_struct_const(pCreateInfo->pNext, NATIVE_BUFFER_ANDROID);
 
-   const struct v3dv_format *format = v3dv_get_format(pCreateInfo->format);
+   int native_buf_fd = -1;
+   int native_buf_stride = 0;
+   int native_buf_size = 0;
+
+   if (native_buffer != NULL) {
+      VkResult result = v3dv_gralloc_info(device, native_buffer, &native_buf_fd,
+                                          &native_buf_stride, &native_buf_size,
+                                          &modifier);
+      if (result != VK_SUCCESS)
+         return result;
+
+      if (modifier != DRM_FORMAT_MOD_BROADCOM_UIF)
+         tiling = VK_IMAGE_TILING_LINEAR;
+   }
+#endif
+
+   const struct v3dv_format *format =
+      v3dv_X(device, get_format)(pCreateInfo->format);
    v3dv_assert(format != NULL && format->supported);
 
-   image = vk_object_zalloc(&device->vk, pAllocator, sizeof(*image),
-                            VK_OBJECT_TYPE_IMAGE);
-   if (!image)
-      return vk_error(device->instance, VK_ERROR_OUT_OF_HOST_MEMORY);
-
    assert(pCreateInfo->samples == VK_SAMPLE_COUNT_1_BIT ||
           pCreateInfo->samples == VK_SAMPLE_COUNT_4_BIT);
 
-   image->type = pCreateInfo->imageType;
-   image->extent = pCreateInfo->extent;
-   image->vk_format = pCreateInfo->format;
    image->format = format;
-   image->aspects = vk_format_aspects(image->vk_format);
-   image->levels = pCreateInfo->mipLevels;
-   image->array_size = pCreateInfo->arrayLayers;
-   image->samples = pCreateInfo->samples;
-   image->usage = pCreateInfo->usage;
-   image->flags = pCreateInfo->flags;
+   image->cpp = vk_format_get_blocksize(image->vk.format);
+   image->tiled = tiling == VK_IMAGE_TILING_OPTIMAL ||
+                  (tiling == VK_IMAGE_TILING_DRM_FORMAT_MODIFIER_EXT &&
+                   modifier != DRM_FORMAT_MOD_LINEAR);
 
-   image->drm_format_mod = modifier;
-   image->tiling = tiling;
-   image->tiled = tiling == VK_IMAGE_TILING_OPTIMAL;
+   image->vk.tiling = tiling;
+   image->vk.drm_format_mod = modifier;
 
-   image->cpp = vk_format_get_blocksize(image->vk_format);
+   /* Our meta paths can create image views with compatible formats for any
+    * image, so always set this flag to keep the common Vulkan image code
+    * happy.
+    */
+   image->vk.create_flags |= VK_IMAGE_CREATE_MUTABLE_FORMAT_BIT;
 
    v3d_setup_slices(image);
 
+#ifdef ANDROID
+   if (native_buffer != NULL) {
+      image->slices[0].stride = native_buf_stride;
+      image->slices[0].size = image->size = native_buf_size;
+
+      VkResult result = v3dv_import_native_buffer_fd(v3dv_device_to_handle(device),
+                                                     native_buf_fd, pAllocator,
+                                                     v3dv_image_to_handle(image));
+      if (result != VK_SUCCESS)
+         return result;
+   }
+#endif
+
+   return VK_SUCCESS;
+}
+
+static VkResult
+create_image(struct v3dv_device *device,
+             const VkImageCreateInfo *pCreateInfo,
+             const VkAllocationCallbacks *pAllocator,
+             VkImage *pImage)
+{
+   struct v3dv_image *image = NULL;
+
+   image = vk_image_create(&device->vk, pCreateInfo, pAllocator, sizeof(*image));
+   if (image == NULL)
+      return vk_error(device, VK_ERROR_OUT_OF_HOST_MEMORY);
+
+   VkResult result = v3dv_image_init(device, pCreateInfo, pAllocator, image);
+   if (result != VK_SUCCESS) {
+      vk_image_destroy(&device->vk, pAllocator, &image->vk);
+      return result;
+   }
+
    *pImage = v3dv_image_to_handle(image);
 
    return VK_SUCCESS;
 }
 
-void
+static VkResult
+create_image_from_swapchain(struct v3dv_device *device,
+                            const VkImageCreateInfo *pCreateInfo,
+                            const VkImageSwapchainCreateInfoKHR *swapchain_info,
+                            const VkAllocationCallbacks *pAllocator,
+                            VkImage *pImage)
+{
+   struct v3dv_image *swapchain_image =
+      v3dv_wsi_get_image_from_swapchain(swapchain_info->swapchain, 0);
+   assert(swapchain_image);
+
+   VkImageCreateInfo local_create_info = *pCreateInfo;
+   local_create_info.pNext = NULL;
+
+   /* Added by wsi code. */
+   local_create_info.usage |= VK_IMAGE_USAGE_COLOR_ATTACHMENT_BIT;
+
+   /* The spec requires TILING_OPTIMAL as input, but the swapchain image may
+    * privately use a different tiling.  See spec anchor
+    * #swapchain-wsi-image-create-info .
+    */
+   assert(local_create_info.tiling == VK_IMAGE_TILING_OPTIMAL);
+   local_create_info.tiling = swapchain_image->vk.tiling;
+
+   VkImageDrmFormatModifierListCreateInfoEXT local_modifier_info = {
+      .sType = VK_STRUCTURE_TYPE_IMAGE_DRM_FORMAT_MODIFIER_LIST_CREATE_INFO_EXT,
+      .drmFormatModifierCount = 1,
+      .pDrmFormatModifiers = &swapchain_image->vk.drm_format_mod,
+   };
+
+   if (swapchain_image->vk.drm_format_mod != DRM_FORMAT_MOD_INVALID)
+      __vk_append_struct(&local_create_info, &local_modifier_info);
+
+   assert(swapchain_image->vk.image_type == local_create_info.imageType);
+   assert(swapchain_image->vk.format == local_create_info.format);
+   assert(swapchain_image->vk.extent.width == local_create_info.extent.width);
+   assert(swapchain_image->vk.extent.height == local_create_info.extent.height);
+   assert(swapchain_image->vk.extent.depth == local_create_info.extent.depth);
+   assert(swapchain_image->vk.array_layers == local_create_info.arrayLayers);
+   assert(swapchain_image->vk.samples == local_create_info.samples);
+   assert(swapchain_image->vk.tiling == local_create_info.tiling);
+   assert((swapchain_image->vk.usage & local_create_info.usage) ==
+          local_create_info.usage);
+
+   return create_image(device, &local_create_info, pAllocator, pImage);
+}
+
+VKAPI_ATTR VkResult VKAPI_CALL
+v3dv_CreateImage(VkDevice _device,
+                 const VkImageCreateInfo *pCreateInfo,
+                 const VkAllocationCallbacks *pAllocator,
+                 VkImage *pImage)
+{
+   V3DV_FROM_HANDLE(v3dv_device, device, _device);
+
+#ifdef ANDROID
+   /* VkImageSwapchainCreateInfoKHR is not useful at all */
+   const VkImageSwapchainCreateInfoKHR *swapchain_info = NULL;
+#else
+   const VkImageSwapchainCreateInfoKHR *swapchain_info =
+      vk_find_struct_const(pCreateInfo->pNext, IMAGE_SWAPCHAIN_CREATE_INFO_KHR);
+#endif
+
+   if (swapchain_info && swapchain_info->swapchain != VK_NULL_HANDLE)
+      return create_image_from_swapchain(device, pCreateInfo, swapchain_info,
+                                         pAllocator, pImage);
+
+   return create_image(device, pCreateInfo, pAllocator, pImage);
+}
+
+VKAPI_ATTR void VKAPI_CALL
 v3dv_GetImageSubresourceLayout(VkDevice device,
                                VkImage _image,
                                const VkImageSubresource *subresource,
@@ -353,12 +454,13 @@ v3dv_GetImageSubresourceLayout(VkDevice device,
    const struct v3d_resource_slice *slice =
       &image->slices[subresource->mipLevel];
    layout->offset =
-      v3dv_layer_offset(image, subresource->mipLevel, subresource->arrayLayer);
+      v3dv_layer_offset(image, subresource->mipLevel, subresource->arrayLayer) -
+      image->mem_offset;
    layout->rowPitch = slice->stride;
    layout->depthPitch = image->cube_map_stride;
    layout->arrayPitch = image->cube_map_stride;
 
-   if (image->type != VK_IMAGE_TYPE_3D) {
+   if (image->vk.image_type != VK_IMAGE_TYPE_3D) {
       layout->size = slice->size;
    } else {
       /* For 3D images, the size of the slice represents the size of a 2D slice
@@ -368,7 +470,7 @@ v3dv_GetImageSubresourceLayout(VkDevice device,
        * arranged in memory from last to first).
        */
       if (subresource->mipLevel == 0) {
-         layout->size = slice->size * image->extent.depth;
+         layout->size = slice->size * image->vk.extent.depth;
       } else {
             const struct v3d_resource_slice *prev_slice =
                &image->slices[subresource->mipLevel - 1];
@@ -377,23 +479,7 @@ v3dv_GetImageSubresourceLayout(VkDevice device,
    }
 }
 
-VkResult
-v3dv_GetImageDrmFormatModifierPropertiesEXT(
-   VkDevice device,
-   VkImage _image,
-   VkImageDrmFormatModifierPropertiesEXT *pProperties)
-{
-   V3DV_FROM_HANDLE(v3dv_image, image, _image);
-
-   assert(pProperties->sType ==
-          VK_STRUCTURE_TYPE_IMAGE_DRM_FORMAT_MODIFIER_PROPERTIES_EXT);
-
-   pProperties->drmFormatModifier = image->drm_format_mod;
-
-   return VK_SUCCESS;
-}
-
-void
+VKAPI_ATTR void VKAPI_CALL
 v3dv_DestroyImage(VkDevice _device,
                   VkImage _image,
                   const VkAllocationCallbacks* pAllocator)
@@ -404,7 +490,12 @@ v3dv_DestroyImage(VkDevice _device,
    if (image == NULL)
       return;
 
-   vk_object_free(&device->vk, pAllocator, image);
+#ifdef ANDROID
+   if (image->is_native_buffer_memory)
+      v3dv_FreeMemory(_device,  v3dv_device_memory_to_handle(image->mem), pAllocator);
+#endif
+
+   vk_image_destroy(&device->vk, pAllocator, &image->vk);
 }
 
 VkImageViewType
@@ -419,217 +510,25 @@ v3dv_image_type_to_view_type(VkImageType type)
    }
 }
 
-/*
- * This method translates pipe_swizzle to the swizzle values used at the
- * packet TEXTURE_SHADER_STATE
- *
- * FIXME: C&P from v3d, common place?
- */
-static uint32_t
-translate_swizzle(unsigned char pipe_swizzle)
+static VkResult
+create_image_view(struct v3dv_device *device,
+                  bool driver_internal,
+                  const VkImageViewCreateInfo *pCreateInfo,
+                  const VkAllocationCallbacks *pAllocator,
+                  VkImageView *pView)
 {
-   switch (pipe_swizzle) {
-   case PIPE_SWIZZLE_0:
-      return 0;
-   case PIPE_SWIZZLE_1:
-      return 1;
-   case PIPE_SWIZZLE_X:
-   case PIPE_SWIZZLE_Y:
-   case PIPE_SWIZZLE_Z:
-   case PIPE_SWIZZLE_W:
-      return 2 + pipe_swizzle;
-   default:
-      unreachable("unknown swizzle");
-   }
-}
-
-/*
- * Packs and ensure bo for the shader state (the latter can be temporal).
- */
-static void
-pack_texture_shader_state_helper(struct v3dv_device *device,
-                                 struct v3dv_image_view *image_view,
-                                 bool for_cube_map_array_storage)
-{
-   assert(!for_cube_map_array_storage ||
-          image_view->type == VK_IMAGE_VIEW_TYPE_CUBE_ARRAY);
-   const uint32_t index = for_cube_map_array_storage ? 1 : 0;
-
-   assert(image_view->image);
-   const struct v3dv_image *image = image_view->image;
-
-   assert(image->samples == VK_SAMPLE_COUNT_1_BIT ||
-          image->samples == VK_SAMPLE_COUNT_4_BIT);
-   const uint32_t msaa_scale = image->samples == VK_SAMPLE_COUNT_1_BIT ? 1 : 2;
-
-   v3dv_pack(image_view->texture_shader_state[index], TEXTURE_SHADER_STATE, tex) {
-
-      tex.level_0_is_strictly_uif =
-         (image->slices[0].tiling == VC5_TILING_UIF_XOR ||
-          image->slices[0].tiling == VC5_TILING_UIF_NO_XOR);
-
-      tex.level_0_xor_enable = (image->slices[0].tiling == VC5_TILING_UIF_XOR);
-
-      if (tex.level_0_is_strictly_uif)
-         tex.level_0_ub_pad = image->slices[0].ub_pad;
-
-      /* FIXME: v3d never sets uif_xor_disable, but uses it on the following
-       * check so let's set the default value
-       */
-      tex.uif_xor_disable = false;
-      if (tex.uif_xor_disable ||
-          tex.level_0_is_strictly_uif) {
-         tex.extended = true;
-      }
-
-      tex.base_level = image_view->base_level;
-      tex.max_level = image_view->max_level;
-
-      tex.swizzle_r = translate_swizzle(image_view->swizzle[0]);
-      tex.swizzle_g = translate_swizzle(image_view->swizzle[1]);
-      tex.swizzle_b = translate_swizzle(image_view->swizzle[2]);
-      tex.swizzle_a = translate_swizzle(image_view->swizzle[3]);
-
-      tex.texture_type = image_view->format->tex_type;
-
-      if (image->type == VK_IMAGE_TYPE_3D) {
-         tex.image_depth = image->extent.depth;
-      } else {
-         tex.image_depth = (image_view->last_layer - image_view->first_layer) + 1;
-      }
-
-      /* Empirical testing with CTS shows that when we are sampling from cube
-       * arrays we want to set image depth to layers / 6, but not when doing
-       * image load/store.
-       */
-      if (image_view->type == VK_IMAGE_VIEW_TYPE_CUBE_ARRAY &&
-          !for_cube_map_array_storage) {
-         assert(tex.image_depth % 6 == 0);
-         tex.image_depth /= 6;
-      }
-
-      tex.image_height = image->extent.height * msaa_scale;
-      tex.image_width = image->extent.width * msaa_scale;
-
-      /* On 4.x, the height of a 1D texture is redefined to be the
-       * upper 14 bits of the width (which is only usable with txf).
-       */
-      if (image->type == VK_IMAGE_TYPE_1D) {
-         tex.image_height = tex.image_width >> 14;
-      }
-      tex.image_width &= (1 << 14) - 1;
-      tex.image_height &= (1 << 14) - 1;
-
-      tex.array_stride_64_byte_aligned = image->cube_map_stride / 64;
-
-      tex.srgb = vk_format_is_srgb(image_view->vk_format);
-
-      /* At this point we don't have the job. That's the reason the first
-       * parameter is NULL, to avoid a crash when cl_pack_emit_reloc tries to
-       * add the bo to the job. This also means that we need to add manually
-       * the image bo to the job using the texture.
-       */
-      const uint32_t base_offset =
-         image->mem->bo->offset +
-         v3dv_layer_offset(image, 0, image_view->first_layer);
-      tex.texture_base_pointer = v3dv_cl_address(NULL, base_offset);
-   }
-}
-
-static void
-pack_texture_shader_state(struct v3dv_device *device,
-                          struct v3dv_image_view *iview)
-{
-   pack_texture_shader_state_helper(device, iview, false);
-   if (iview->type == VK_IMAGE_VIEW_TYPE_CUBE_ARRAY)
-      pack_texture_shader_state_helper(device, iview, true);
-}
-
-static enum pipe_swizzle
-vk_component_mapping_to_pipe_swizzle(VkComponentSwizzle comp,
-                                     VkComponentSwizzle swz)
-{
-   if (swz == VK_COMPONENT_SWIZZLE_IDENTITY)
-      swz = comp;
-
-   switch (swz) {
-   case VK_COMPONENT_SWIZZLE_ZERO:
-      return PIPE_SWIZZLE_0;
-   case VK_COMPONENT_SWIZZLE_ONE:
-      return PIPE_SWIZZLE_1;
-   case VK_COMPONENT_SWIZZLE_R:
-      return PIPE_SWIZZLE_X;
-   case VK_COMPONENT_SWIZZLE_G:
-      return PIPE_SWIZZLE_Y;
-   case VK_COMPONENT_SWIZZLE_B:
-      return PIPE_SWIZZLE_Z;
-   case VK_COMPONENT_SWIZZLE_A:
-      return PIPE_SWIZZLE_W;
-   default:
-      unreachable("Unknown VkComponentSwizzle");
-   };
-}
-
-VkResult
-v3dv_CreateImageView(VkDevice _device,
-                     const VkImageViewCreateInfo *pCreateInfo,
-                     const VkAllocationCallbacks *pAllocator,
-                     VkImageView *pView)
-{
-   V3DV_FROM_HANDLE(v3dv_device, device, _device);
    V3DV_FROM_HANDLE(v3dv_image, image, pCreateInfo->image);
    struct v3dv_image_view *iview;
 
-   iview = vk_object_zalloc(&device->vk, pAllocator, sizeof(*iview),
-                            VK_OBJECT_TYPE_IMAGE_VIEW);
+   iview = vk_image_view_create(&device->vk, driver_internal, pCreateInfo,
+                                pAllocator, sizeof(*iview));
    if (iview == NULL)
-      return vk_error(device->instance, VK_ERROR_OUT_OF_HOST_MEMORY);
+      return vk_error(device, VK_ERROR_OUT_OF_HOST_MEMORY);
 
    const VkImageSubresourceRange *range = &pCreateInfo->subresourceRange;
 
-   assert(range->layerCount > 0);
-   assert(range->baseMipLevel < image->levels);
-
-#ifdef DEBUG
-   switch (image->type) {
-   case VK_IMAGE_TYPE_1D:
-   case VK_IMAGE_TYPE_2D:
-      assert(range->baseArrayLayer + v3dv_layer_count(image, range) - 1 <=
-             image->array_size);
-      break;
-   case VK_IMAGE_TYPE_3D:
-      assert(range->baseArrayLayer + v3dv_layer_count(image, range) - 1
-             <= u_minify(image->extent.depth, range->baseMipLevel));
-      /* VK_KHR_maintenance1 */
-      assert(pCreateInfo->viewType != VK_IMAGE_VIEW_TYPE_2D ||
-             ((image->flags & VK_IMAGE_CREATE_2D_ARRAY_COMPATIBLE_BIT) &&
-              range->levelCount == 1 && range->layerCount == 1));
-      assert(pCreateInfo->viewType != VK_IMAGE_VIEW_TYPE_2D_ARRAY ||
-             ((image->flags & VK_IMAGE_CREATE_2D_ARRAY_COMPATIBLE_BIT) &&
-              range->levelCount == 1));
-      break;
-   default:
-      unreachable("bad VkImageType");
-   }
-#endif
-
-   iview->image = image;
-   iview->aspects = range->aspectMask;
-   iview->type = pCreateInfo->viewType;
-
-   iview->base_level = range->baseMipLevel;
-   iview->max_level = iview->base_level + v3dv_level_count(image, range) - 1;
-   iview->extent = (VkExtent3D) {
-      .width  = u_minify(image->extent.width , iview->base_level),
-      .height = u_minify(image->extent.height, iview->base_level),
-      .depth  = u_minify(image->extent.depth , iview->base_level),
-   };
-
-   iview->first_layer = range->baseArrayLayer;
-   iview->last_layer = range->baseArrayLayer +
-                       v3dv_layer_count(image, range) - 1;
-   iview->offset =
-      v3dv_layer_offset(image, iview->base_level, iview->first_layer);
+   iview->offset = v3dv_layer_offset(image, iview->vk.base_mip_level,
+                                     iview->vk.base_array_layer);
 
    /* If we have D24S8 format but the view only selects the stencil aspect
     * we want to re-interpret the format as RGBA8_UINT, then map our stencil
@@ -652,45 +551,56 @@ v3dv_CreateImageView(VkDevice _device,
        * util_format_compose_swizzles. Would be good to check if it would be
        * better to reimplement the latter using vk component
        */
-      image_view_swizzle[0] =
-         vk_component_mapping_to_pipe_swizzle(VK_COMPONENT_SWIZZLE_R,
-                                              pCreateInfo->components.r);
-      image_view_swizzle[1] =
-         vk_component_mapping_to_pipe_swizzle(VK_COMPONENT_SWIZZLE_G,
-                                              pCreateInfo->components.g);
-      image_view_swizzle[2] =
-         vk_component_mapping_to_pipe_swizzle(VK_COMPONENT_SWIZZLE_B,
-                                              pCreateInfo->components.b);
-      image_view_swizzle[3] =
-         vk_component_mapping_to_pipe_swizzle(VK_COMPONENT_SWIZZLE_A,
-                                              pCreateInfo->components.a);
+      vk_component_mapping_to_pipe_swizzle(iview->vk.swizzle,
+                                           image_view_swizzle);
    }
 
-   iview->vk_format = format;
-   iview->format = v3dv_get_format(format);
+   iview->vk.view_format = format;
+   iview->format = v3dv_X(device, get_format)(format);
    assert(iview->format && iview->format->supported);
 
-   if (vk_format_is_depth_or_stencil(iview->vk_format)) {
-      iview->internal_type = v3dv_get_internal_depth_type(iview->vk_format);
+   if (vk_format_is_depth_or_stencil(iview->vk.view_format)) {
+      iview->internal_type =
+         v3dv_X(device, get_internal_depth_type)(iview->vk.view_format);
    } else {
-      v3dv_get_internal_type_bpp_for_output_format(iview->format->rt_type,
-                                                   &iview->internal_type,
-                                                   &iview->internal_bpp);
+      v3dv_X(device, get_internal_type_bpp_for_output_format)
+         (iview->format->rt_type, &iview->internal_type, &iview->internal_bpp);
    }
 
-   const uint8_t *format_swizzle = v3dv_get_format_swizzle(format);
+   const uint8_t *format_swizzle = v3dv_get_format_swizzle(device, format);
    util_format_compose_swizzles(format_swizzle, image_view_swizzle,
                                 iview->swizzle);
-   iview->swap_rb = iview->swizzle[0] == PIPE_SWIZZLE_Z;
 
-   pack_texture_shader_state(device, iview);
+   iview->swap_rb = v3dv_format_swizzle_needs_rb_swap(format_swizzle);
+   iview->channel_reverse = v3dv_format_swizzle_needs_reverse(format_swizzle);
+
+   v3dv_X(device, pack_texture_shader_state)(device, iview);
 
    *pView = v3dv_image_view_to_handle(iview);
 
    return VK_SUCCESS;
 }
 
-void
+VkResult
+v3dv_create_image_view(struct v3dv_device *device,
+                       const VkImageViewCreateInfo *pCreateInfo,
+                       VkImageView *pView)
+{
+   return create_image_view(device, true, pCreateInfo, NULL, pView);
+}
+
+VKAPI_ATTR VkResult VKAPI_CALL
+v3dv_CreateImageView(VkDevice _device,
+                     const VkImageViewCreateInfo *pCreateInfo,
+                     const VkAllocationCallbacks *pAllocator,
+                     VkImageView *pView)
+{
+   V3DV_FROM_HANDLE(v3dv_device, device, _device);
+
+   return create_image_view(device, false, pCreateInfo, pAllocator, pView);
+}
+
+VKAPI_ATTR void VKAPI_CALL
 v3dv_DestroyImageView(VkDevice _device,
                       VkImageView imageView,
                       const VkAllocationCallbacks* pAllocator)
@@ -701,52 +611,10 @@ v3dv_DestroyImageView(VkDevice _device,
    if (image_view == NULL)
       return;
 
-   vk_object_free(&device->vk, pAllocator, image_view);
+   vk_image_view_destroy(&device->vk, pAllocator, &image_view->vk);
 }
 
-static void
-pack_texture_shader_state_from_buffer_view(struct v3dv_device *device,
-                                           struct v3dv_buffer_view *buffer_view)
-{
-   assert(buffer_view->buffer);
-   const struct v3dv_buffer *buffer = buffer_view->buffer;
-
-   v3dv_pack(buffer_view->texture_shader_state, TEXTURE_SHADER_STATE, tex) {
-      tex.swizzle_r = translate_swizzle(PIPE_SWIZZLE_X);
-      tex.swizzle_g = translate_swizzle(PIPE_SWIZZLE_Y);
-      tex.swizzle_b = translate_swizzle(PIPE_SWIZZLE_Z);
-      tex.swizzle_a = translate_swizzle(PIPE_SWIZZLE_W);
-
-      tex.image_depth = 1;
-
-      /* On 4.x, the height of a 1D texture is redefined to be the upper 14
-       * bits of the width (which is only usable with txf) (or in other words,
-       * we are providing a 28 bit field for size, but split on the usual
-       * 14bit height/width).
-       */
-      tex.image_width = buffer_view->num_elements;
-      tex.image_height = tex.image_width >> 14;
-      tex.image_width &= (1 << 14) - 1;
-      tex.image_height &= (1 << 14) - 1;
-
-      tex.texture_type = buffer_view->format->tex_type;
-      tex.srgb = vk_format_is_srgb(buffer_view->vk_format);
-
-      /* At this point we don't have the job. That's the reason the first
-       * parameter is NULL, to avoid a crash when cl_pack_emit_reloc tries to
-       * add the bo to the job. This also means that we need to add manually
-       * the image bo to the job using the texture.
-       */
-      const uint32_t base_offset =
-         buffer->mem->bo->offset +
-         buffer->mem_offset +
-         buffer_view->offset;
-
-      tex.texture_base_pointer = v3dv_cl_address(NULL, base_offset);
-   }
-}
-
-VkResult
+VKAPI_ATTR VkResult VKAPI_CALL
 v3dv_CreateBufferView(VkDevice _device,
                       const VkBufferViewCreateInfo *pCreateInfo,
                       const VkAllocationCallbacks *pAllocator,
@@ -754,14 +622,14 @@ v3dv_CreateBufferView(VkDevice _device,
 {
    V3DV_FROM_HANDLE(v3dv_device, device, _device);
 
-   const struct v3dv_buffer *buffer =
+   struct v3dv_buffer *buffer =
       v3dv_buffer_from_handle(pCreateInfo->buffer);
 
    struct v3dv_buffer_view *view =
       vk_object_zalloc(&device->vk, pAllocator, sizeof(*view),
                        VK_OBJECT_TYPE_BUFFER_VIEW);
    if (!view)
-      return vk_error(device->instance, VK_ERROR_OUT_OF_HOST_MEMORY);
+      return vk_error(device, VK_ERROR_OUT_OF_HOST_MEMORY);
 
    uint32_t range;
    if (pCreateInfo->range == VK_WHOLE_SIZE)
@@ -777,22 +645,21 @@ v3dv_CreateBufferView(VkDevice _device,
    view->size = view->offset + range;
    view->num_elements = num_elements;
    view->vk_format = pCreateInfo->format;
-   view->format = v3dv_get_format(view->vk_format);
+   view->format = v3dv_X(device, get_format)(view->vk_format);
 
-   v3dv_get_internal_type_bpp_for_output_format(view->format->rt_type,
-                                                &view->internal_type,
-                                                &view->internal_bpp);
+   v3dv_X(device, get_internal_type_bpp_for_output_format)
+      (view->format->rt_type, &view->internal_type, &view->internal_bpp);
 
    if (buffer->usage & VK_BUFFER_USAGE_UNIFORM_TEXEL_BUFFER_BIT ||
        buffer->usage & VK_BUFFER_USAGE_STORAGE_TEXEL_BUFFER_BIT)
-      pack_texture_shader_state_from_buffer_view(device, view);
+      v3dv_X(device, pack_texture_shader_state_from_buffer_view)(device, view);
 
    *pView = v3dv_buffer_view_to_handle(view);
 
    return VK_SUCCESS;
 }
 
-void
+VKAPI_ATTR void VKAPI_CALL
 v3dv_DestroyBufferView(VkDevice _device,
                        VkBufferView bufferView,
                        const VkAllocationCallbacks *pAllocator)
