@@ -33,51 +33,10 @@
 struct gl_linked_shader;
 struct gl_shader_program;
 
-/* Operations for lower_instructions() */
-#define SUB_TO_ADD_NEG     0x01
-#define LDEXP_TO_ARITH     0x80
-#define CARRY_TO_ARITH     0x100
-#define BORROW_TO_ARITH    0x200
-#define DOPS_TO_DFRAC      0x800
-#define DFREXP_DLDEXP_TO_ARITH    0x1000
-#define BIT_COUNT_TO_MATH         0x02000
-#define EXTRACT_TO_SHIFTS         0x04000
-#define INSERT_TO_SHIFTS          0x08000
-#define REVERSE_TO_SHIFTS         0x10000
-#define FIND_LSB_TO_FLOAT_CAST    0x20000
-#define FIND_MSB_TO_FLOAT_CAST    0x40000
-#define IMUL_HIGH_TO_MUL          0x80000
-#define SQRT_TO_ABS_SQRT          0x200000
-
 /* Operations for lower_64bit_integer_instructions() */
 #define DIV64                     (1U << 0)
 #define MOD64                     (1U << 1)
 
-/**
- * \see class lower_packing_builtins_visitor
- */
-enum lower_packing_builtins_op {
-   LOWER_PACK_UNPACK_NONE               = 0x0000,
-
-   LOWER_PACK_SNORM_2x16                = 0x0001,
-   LOWER_UNPACK_SNORM_2x16              = 0x0002,
-
-   LOWER_PACK_UNORM_2x16                = 0x0004,
-   LOWER_UNPACK_UNORM_2x16              = 0x0008,
-
-   LOWER_PACK_HALF_2x16                 = 0x0010,
-   LOWER_UNPACK_HALF_2x16               = 0x0020,
-
-   LOWER_PACK_SNORM_4x8                 = 0x0040,
-   LOWER_UNPACK_SNORM_4x8               = 0x0080,
-
-   LOWER_PACK_UNORM_4x8                 = 0x0100,
-   LOWER_UNPACK_UNORM_4x8               = 0x0200,
-
-   LOWER_PACK_USE_BFI                   = 0x0400,
-   LOWER_PACK_USE_BFE                   = 0x0800,
-};
-
 bool do_common_optimization(exec_list *ir, bool linked,
                             const struct gl_shader_compiler_options *options,
                             bool native_integers);
@@ -103,27 +62,26 @@ bool do_if_simplification(exec_list *instructions);
 bool opt_flatten_nested_if_blocks(exec_list *instructions);
 bool do_mat_op_to_vec(exec_list *instructions);
 bool do_minmax_prune(exec_list *instructions);
-bool do_structure_splitting(exec_list *instructions);
 bool do_tree_grafting(exec_list *instructions);
 bool do_vec_index_to_cond_assign(exec_list *instructions);
-bool do_vec_index_to_swizzle(exec_list *instructions);
 bool lower_discard(exec_list *instructions);
 void lower_discard_flow(exec_list *instructions);
-bool lower_instructions(exec_list *instructions, unsigned what_to_lower);
+bool lower_instructions(exec_list *instructions, bool have_ldexp,
+                        bool have_dfrexp, bool have_dround,
+                        bool force_abs_sqrt, bool have_gpu_shader5);
 bool lower_clip_cull_distance(struct gl_shader_program *prog,
                               gl_linked_shader *shader);
-void lower_output_reads(unsigned stage, exec_list *instructions);
-bool lower_packing_builtins(exec_list *instructions, int op_mask);
+bool lower_packing_builtins(exec_list *instructions,
+                            bool has_shading_language_packing,
+                            bool has_gpu_shader5,
+                            bool has_half_float_packing);
 bool lower_vector_insert(exec_list *instructions, bool lower_nonconstant_index);
 bool lower_vector_derefs(gl_linked_shader *shader);
 void lower_named_interface_blocks(void *mem_ctx, gl_linked_shader *shader);
-bool optimize_split_arrays(exec_list *instructions, bool linked);
 void optimize_dead_builtin_variables(exec_list *instructions,
                                      enum ir_variable_mode other);
 bool lower_tess_level(gl_linked_shader *shader);
 
-bool lower_vertex_id(gl_linked_shader *shader);
-bool lower_cs_derived(gl_linked_shader *shader);
 bool lower_blend_equation_advanced(gl_linked_shader *shader, bool coherent);
 
 bool lower_builtins(exec_list *instructions);
