@@ -30,12 +30,12 @@
 #include "etnaviv_priv.h"
 #include "etnaviv_drmif.h"
 
-simple_mtx_t etna_drm_table_lock = _SIMPLE_MTX_INITIALIZER_NP;
+simple_mtx_t etna_device_lock = SIMPLE_MTX_INITIALIZER;
 
 /* set buffer name, and add to table, call w/ etna_drm_table_lock held: */
 static void set_name(struct etna_bo *bo, uint32_t name)
 {
-	simple_mtx_assert_locked(&etna_drm_table_lock);
+	simple_mtx_assert_locked(&etna_device_lock);
 
 	bo->name = name;
 	/* add ourself into the name table: */
@@ -56,7 +56,7 @@ static void _etna_bo_free(struct etna_bo *bo)
 	DEBUG_BO("Del bo:", bo);
 	VG_BO_FREE(bo);
 
-	simple_mtx_assert_locked(&etna_drm_table_lock);
+	simple_mtx_assert_locked(&etna_device_lock);
 
 	if (bo->va)
 		util_vma_heap_free(&bo->dev->address_space, bo->va, bo->size);
@@ -81,7 +81,7 @@ static void _etna_bo_free(struct etna_bo *bo)
 
 void etna_bo_kill_zombies(struct etna_device *dev)
 {
-	simple_mtx_assert_locked(&etna_drm_table_lock);
+	simple_mtx_assert_locked(&etna_device_lock);
 
 	list_for_each_entry_safe(struct etna_bo, bo, &dev->zombie_list, list) {
 		VG_BO_OBTAIN(bo);
@@ -93,7 +93,7 @@ void etna_bo_kill_zombies(struct etna_device *dev)
 
 static void etna_bo_cleanup_zombies(struct etna_device *dev)
 {
-	simple_mtx_assert_locked(&etna_drm_table_lock);
+	simple_mtx_assert_locked(&etna_device_lock);
 
 	list_for_each_entry_safe(struct etna_bo, bo, &dev->zombie_list, list) {
 		/* Stop once we reach a busy BO - all others past this point were
@@ -130,7 +130,7 @@ static struct etna_bo *lookup_bo(void *tbl, uint32_t handle)
 	struct etna_bo *bo = NULL;
 	struct hash_entry *entry;
 
-	simple_mtx_assert_locked(&etna_drm_table_lock);
+	simple_mtx_assert_locked(&etna_device_lock);
 
 	entry = _mesa_hash_table_search(tbl, &handle);
 
@@ -155,7 +155,7 @@ static struct etna_bo *bo_from_handle(struct etna_device *dev,
 {
 	struct etna_bo *bo = calloc(sizeof(*bo), 1);
 
-	simple_mtx_assert_locked(&etna_drm_table_lock);
+	simple_mtx_assert_locked(&etna_device_lock);
 
 	if (!bo) {
 		struct drm_gem_close req = {
@@ -202,10 +202,10 @@ struct etna_bo *etna_bo_new(struct etna_device *dev, uint32_t size,
 	if (ret)
 		return NULL;
 
-	simple_mtx_lock(&etna_drm_table_lock);
+	simple_mtx_lock(&etna_device_lock);
 	bo = bo_from_handle(dev, size, req.handle, flags);
 	bo->reuse = 1;
-	simple_mtx_unlock(&etna_drm_table_lock);
+	simple_mtx_unlock(&etna_device_lock);
 
 	DEBUG_BO("New bo:", bo);
 	VG_BO_ALLOC(bo);
@@ -220,26 +220,6 @@ struct etna_bo *etna_bo_ref(struct etna_bo *bo)
 	return bo;
 }
 
-/* get buffer info */
-static int get_buffer_info(struct etna_bo *bo)
-{
-	int ret;
-	struct drm_etnaviv_gem_info req = {
-		.handle = bo->handle,
-	};
-
-	ret = drmCommandWriteRead(bo->dev->fd, DRM_ETNAVIV_GEM_INFO,
-			&req, sizeof(req));
-	if (ret) {
-		return ret;
-	}
-
-	/* really all we need for now is mmap offset */
-	bo->offset = req.offset;
-
-	return 0;
-}
-
 /* import a buffer object from DRI2 name */
 struct etna_bo *etna_bo_from_name(struct etna_device *dev,
 		uint32_t name)
@@ -249,7 +229,7 @@ struct etna_bo *etna_bo_from_name(struct etna_device *dev,
 		.name = name,
 	};
 
-	simple_mtx_lock(&etna_drm_table_lock);
+	simple_mtx_lock(&etna_device_lock);
 
 	/* check name table first, to see if bo is already open: */
 	bo = lookup_bo(dev->name_table, name);
@@ -273,7 +253,7 @@ struct etna_bo *etna_bo_from_name(struct etna_device *dev,
 	}
 
 out_unlock:
-	simple_mtx_unlock(&etna_drm_table_lock);
+	simple_mtx_unlock(&etna_device_lock);
 
 	return bo;
 }
@@ -292,11 +272,11 @@ struct etna_bo *etna_bo_from_dmabuf(struct etna_device *dev, int fd)
 	 * racing against etna_bo_del, which might invalidate the
 	 * returned handle.
 	 */
-	simple_mtx_lock(&etna_drm_table_lock);
+	simple_mtx_lock(&etna_device_lock);
 
 	ret = drmPrimeFDToHandle(dev->fd, fd, &handle);
 	if (ret) {
-		simple_mtx_unlock(&etna_drm_table_lock);
+		simple_mtx_unlock(&etna_device_lock);
 		return NULL;
 	}
 
@@ -314,7 +294,7 @@ struct etna_bo *etna_bo_from_dmabuf(struct etna_device *dev, int fd)
 	VG_BO_ALLOC(bo);
 
 out_unlock:
-	simple_mtx_unlock(&etna_drm_table_lock);
+	simple_mtx_unlock(&etna_device_lock);
 
 	return bo;
 }
@@ -327,7 +307,7 @@ void etna_bo_del(struct etna_bo *bo)
 
 	struct etna_device *dev = bo->dev;
 
-	simple_mtx_lock(&etna_drm_table_lock);
+	simple_mtx_lock(&etna_device_lock);
 
 	/* Must test under table lock to avoid racing with the from_dmabuf/name
 	 * paths, which rely on the BO refcount to be stable over the lookup, so
@@ -342,7 +322,7 @@ void etna_bo_del(struct etna_bo *bo)
 	etna_bo_free(bo);
 	etna_device_del_locked(dev);
 out:
-	simple_mtx_unlock(&etna_drm_table_lock);
+	simple_mtx_unlock(&etna_device_lock);
 }
 
 /* get the global flink/DRI2 buffer name */
@@ -359,9 +339,9 @@ int etna_bo_get_name(struct etna_bo *bo, uint32_t *name)
 			return ret;
 		}
 
-		simple_mtx_lock(&etna_drm_table_lock);
+		simple_mtx_lock(&etna_device_lock);
 		set_name(bo, req.name);
-		simple_mtx_unlock(&etna_drm_table_lock);
+		simple_mtx_unlock(&etna_device_lock);
 		bo->reuse = 0;
 	}
 
@@ -407,16 +387,26 @@ uint32_t etna_bo_gpu_va(struct etna_bo *bo)
 void *etna_bo_map(struct etna_bo *bo)
 {
 	if (!bo->map) {
-		if (!bo->offset) {
-			get_buffer_info(bo);
+		int ret;
+		void *map;
+		struct drm_etnaviv_gem_info req = {
+			.handle = bo->handle,
+		};
+
+		ret = drmCommandWriteRead(bo->dev->fd, DRM_ETNAVIV_GEM_INFO,
+					&req, sizeof(req));
+		if (ret)
+			return NULL;
+
+		map = os_mmap(0, bo->size, PROT_READ | PROT_WRITE,
+				  MAP_SHARED, bo->dev->fd, req.offset);
+		if (map == MAP_FAILED) {
+			ERROR_MSG("mmap failed: %s", strerror(errno));
+			return NULL;
 		}
 
-		bo->map = os_mmap(0, bo->size, PROT_READ | PROT_WRITE,
-				  MAP_SHARED, bo->dev->fd, bo->offset);
-		if (bo->map == MAP_FAILED) {
-			ERROR_MSG("mmap failed: %s", strerror(errno));
-			bo->map = NULL;
-		}
+		if (p_atomic_cmpxchg(&bo->map, NULL, map))
+			munmap(map, bo->size);
 	}
 
 	return bo->map;
