@@ -70,7 +70,7 @@ void v3d_disk_cache_init(struct v3d_screen *screen)
         char timestamp[41];
         _mesa_sha1_format(timestamp, id_sha1);
 
-        screen->disk_cache = disk_cache_create(renderer, timestamp, 0);
+        screen->disk_cache = disk_cache_create(renderer, timestamp, v3d_mesa_debug);
 
         free(renderer);
 }
@@ -124,7 +124,7 @@ v3d_disk_cache_retrieve(struct v3d_context *v3d,
         size_t buffer_size;
         void *buffer = disk_cache_get(cache, cache_key, &buffer_size);
 
-        if (unlikely(V3D_DEBUG & V3D_DEBUG_CACHE)) {
+        if (V3D_DBG(CACHE)) {
                 char sha1[41];
                 _mesa_sha1_format(sha1, cache_key);
                 fprintf(stderr, "[v3d on-disk cache] %s %s\n",
@@ -205,7 +205,7 @@ v3d_disk_cache_store(struct v3d_context *v3d,
         cache_key cache_key;
         v3d_disk_cache_compute_key(cache, key, cache_key);
 
-        if (unlikely(V3D_DEBUG & V3D_DEBUG_CACHE)) {
+        if (V3D_DBG(CACHE)) {
                 char sha1[41];
                 _mesa_sha1_format(sha1, cache_key);
                 fprintf(stderr, "[v3d on-disk cache] storing %s\n", sha1);
