@@ -29,6 +29,7 @@
 
 #include "c99_alloca.h"
 
+#include "api_exec_decl.h"
 #include "main/glthread_marshal.h"
 #include "main/dispatch.h"
 #include "main/varray.h"
@@ -69,7 +70,7 @@ upload_indices(struct gl_context *ctx, unsigned count, unsigned index_size,
    assert(count);
 
    _mesa_glthread_upload(ctx, *indices, index_size * count,
-                         &upload_offset, &upload_buffer, NULL);
+                         &upload_offset, &upload_buffer, NULL, 0);
    assert(upload_buffer);
    *indices = (const GLvoid*)(intptr_t)upload_offset;
 
@@ -89,7 +90,7 @@ upload_multi_indices(struct gl_context *ctx, unsigned total_count,
    assert(total_count);
 
    _mesa_glthread_upload(ctx, NULL, index_size * total_count,
-                         &upload_offset, &upload_buffer, &upload_ptr);
+                         &upload_offset, &upload_buffer, &upload_ptr, 0);
    assert(upload_buffer);
 
    for (unsigned i = 0, offset = 0; i < draw_count; i++) {
@@ -188,10 +189,14 @@ upload_vertices(struct gl_context *ctx, unsigned user_buffer_mask,
          end = end_offset[binding_index];
          assert(start < end);
 
+         /* If the draw start index is non-zero, glthread can upload to offset 0,
+         * which means the attrib offset has to be -(first * stride).
+         * So use signed vertex buffer offsets when possible to save memory.
+         */
          const void *ptr = vao->Attrib[binding_index].Pointer;
          _mesa_glthread_upload(ctx, (uint8_t*)ptr + start,
                                end - start, &upload_offset,
-                               &upload_buffer, NULL);
+                               &upload_buffer, NULL, ctx->Const.VertexBufferOffsetIsInt32 ? 0 : start);
          assert(upload_buffer);
 
          buffers[num_buffers].buffer = upload_buffer;
@@ -239,9 +244,14 @@ upload_vertices(struct gl_context *ctx, unsigned user_buffer_mask,
          size = stride * (num_vertices - 1) + element_size;
       }
 
+      /* If the draw start index is non-zero, glthread can upload to offset 0,
+       * which means the attrib offset has to be -(first * stride).
+       * So use signed vertex buffer offsets when possible to save memory.
+       */
       const void *ptr = vao->Attrib[binding_index].Pointer;
       _mesa_glthread_upload(ctx, (uint8_t*)ptr + offset,
-                            size, &upload_offset, &upload_buffer, NULL);
+                            size, &upload_offset, &upload_buffer, NULL,
+                            ctx->Const.VertexBufferOffsetIsInt32 ? 0 : offset);
       assert(upload_buffer);
 
       buffers[num_buffers].buffer = upload_buffer;
@@ -253,8 +263,31 @@ upload_vertices(struct gl_context *ctx, unsigned user_buffer_mask,
    return true;
 }
 
-/* Generic DrawArrays structure NOT supporting user buffers. Ignore the name. */
+/* DrawArrays without user buffers. */
 struct marshal_cmd_DrawArrays
+{
+   struct marshal_cmd_base cmd_base;
+   GLenum mode;
+   GLint first;
+   GLsizei count;
+};
+
+uint32_t
+_mesa_unmarshal_DrawArrays(struct gl_context *ctx,
+                           const struct marshal_cmd_DrawArrays *cmd)
+{
+   const GLenum mode = cmd->mode;
+   const GLint first = cmd->first;
+   const GLsizei count = cmd->count;
+
+   CALL_DrawArrays(ctx->CurrentServerDispatch, (mode, first, count));
+   const unsigned cmd_size = align(sizeof(*cmd), 8) / 8;
+   assert(cmd_size == cmd->cmd_base.cmd_size);
+   return cmd_size;
+}
+
+/* DrawArraysInstancedBaseInstance without user buffers. */
+struct marshal_cmd_DrawArraysInstancedBaseInstance
 {
    struct marshal_cmd_base cmd_base;
    GLenum mode;
@@ -265,15 +298,9 @@ struct marshal_cmd_DrawArrays
 };
 
 uint32_t
-_mesa_unmarshal_DrawArrays(struct gl_context *ctx,
-                           const struct marshal_cmd_DrawArrays *cmd,
-                           const uint64_t *last)
+_mesa_unmarshal_DrawArraysInstancedBaseInstance(struct gl_context *ctx,
+                                                const struct marshal_cmd_DrawArraysInstancedBaseInstance *cmd)
 {
-   /* Ignore the function name. We use DISPATCH_CMD_DrawArrays
-    * for all DrawArrays variants without user buffers, and
-    * DISPATCH_CMD_DrawArraysInstancedBaseInstance for all DrawArrays
-    * variants with user buffrs.
-    */
    const GLenum mode = cmd->mode;
    const GLint first = cmd->first;
    const GLsizei count = cmd->count;
@@ -283,26 +310,38 @@ _mesa_unmarshal_DrawArrays(struct gl_context *ctx,
    CALL_DrawArraysInstancedBaseInstance(ctx->CurrentServerDispatch,
                                         (mode, first, count, instance_count,
                                          baseinstance));
-   return cmd->cmd_base.cmd_size;
+   const unsigned cmd_size = align(sizeof(*cmd), 8) / 8;
+   assert(cmd_size == cmd->cmd_base.cmd_size);
+   return cmd_size;
 }
 
 static ALWAYS_INLINE void
 draw_arrays_async(struct gl_context *ctx, GLenum mode, GLint first,
                   GLsizei count, GLsizei instance_count, GLuint baseinstance)
 {
-   int cmd_size = sizeof(struct marshal_cmd_DrawArrays);
-   struct marshal_cmd_DrawArrays *cmd =
-      _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawArrays, cmd_size);
+   if (instance_count == 1 && baseinstance == 0) {
+      int cmd_size = sizeof(struct marshal_cmd_DrawArrays);
+      struct marshal_cmd_DrawArrays *cmd =
+         _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawArrays, cmd_size);
 
-   cmd->mode = mode;
-   cmd->first = first;
-   cmd->count = count;
-   cmd->instance_count = instance_count;
-   cmd->baseinstance = baseinstance;
+      cmd->mode = mode;
+      cmd->first = first;
+      cmd->count = count;
+   } else {
+      int cmd_size = sizeof(struct marshal_cmd_DrawArraysInstancedBaseInstance);
+      struct marshal_cmd_DrawArraysInstancedBaseInstance *cmd =
+         _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawArraysInstancedBaseInstance, cmd_size);
+
+      cmd->mode = mode;
+      cmd->first = first;
+      cmd->count = count;
+      cmd->instance_count = instance_count;
+      cmd->baseinstance = baseinstance;
+   }
 }
 
-/* Generic DrawArrays structure supporting user buffers. Ignore the name. */
-struct marshal_cmd_DrawArraysInstancedBaseInstance
+/* DrawArraysInstancedBaseInstance with user buffers. */
+struct marshal_cmd_DrawArraysUserBuf
 {
    struct marshal_cmd_base cmd_base;
    GLenum mode;
@@ -314,15 +353,9 @@ struct marshal_cmd_DrawArraysInstancedBaseInstance
 };
 
 uint32_t
-_mesa_unmarshal_DrawArraysInstancedBaseInstance(struct gl_context *ctx,
-                                                const struct marshal_cmd_DrawArraysInstancedBaseInstance *cmd,
-                                                const uint64_t *last)
+_mesa_unmarshal_DrawArraysUserBuf(struct gl_context *ctx,
+                                  const struct marshal_cmd_DrawArraysUserBuf *cmd)
 {
-   /* Ignore the function name. We use DISPATCH_CMD_DrawArrays
-    * for all DrawArrays variants without user buffers, and
-    * DISPATCH_CMD_DrawArraysInstancedBaseInstance for all DrawArrays
-    * variants with user buffrs.
-    */
    const GLenum mode = cmd->mode;
    const GLint first = cmd->first;
    const GLsizei count = cmd->count;
@@ -357,11 +390,11 @@ draw_arrays_async_user(struct gl_context *ctx, GLenum mode, GLint first,
                        const struct glthread_attrib_binding *buffers)
 {
    int buffers_size = util_bitcount(user_buffer_mask) * sizeof(buffers[0]);
-   int cmd_size = sizeof(struct marshal_cmd_DrawArraysInstancedBaseInstance) +
+   int cmd_size = sizeof(struct marshal_cmd_DrawArraysUserBuf) +
                   buffers_size;
-   struct marshal_cmd_DrawArraysInstancedBaseInstance *cmd;
+   struct marshal_cmd_DrawArraysUserBuf *cmd;
 
-   cmd = _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawArraysInstancedBaseInstance,
+   cmd = _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawArraysUserBuf,
                                          cmd_size);
    cmd->mode = mode;
    cmd->first = first;
@@ -396,14 +429,16 @@ draw_arrays(GLenum mode, GLint first, GLsizei count, GLsizei instance_count,
     * for possible GL errors.
     */
    if (ctx->API == API_OPENGL_CORE || !user_buffer_mask ||
-       count <= 0 || instance_count <= 0) {
+       count <= 0 || instance_count <= 0 ||
+       /* This will just generate GL_INVALID_OPERATION, as it should. */
+       (!compiled_into_dlist && ctx->GLThread.ListMode)) {
       draw_arrays_async(ctx, mode, first, count, instance_count, baseinstance);
       return;
    }
 
    /* Upload and draw. */
    struct glthread_attrib_binding buffers[VERT_ATTRIB_MAX];
-   if (!ctx->GLThread.SupportsNonVBOUploads ||
+   if (!ctx->GLThread.SupportsBufferUploads ||
        !upload_vertices(ctx, user_buffer_mask, first, count, baseinstance,
                         instance_count, buffers)) {
       _mesa_glthread_finish_before(ctx, "DrawArrays");
@@ -417,7 +452,8 @@ draw_arrays(GLenum mode, GLint first, GLsizei count, GLsizei instance_count,
                           user_buffer_mask, buffers);
 }
 
-struct marshal_cmd_MultiDrawArrays
+/* MultiDrawArrays with user buffers. */
+struct marshal_cmd_MultiDrawArraysUserBuf
 {
    struct marshal_cmd_base cmd_base;
    GLenum mode;
@@ -426,9 +462,8 @@ struct marshal_cmd_MultiDrawArrays
 };
 
 uint32_t
-_mesa_unmarshal_MultiDrawArrays(struct gl_context *ctx,
-                                const struct marshal_cmd_MultiDrawArrays *cmd,
-                                const uint64_t *last)
+_mesa_unmarshal_MultiDrawArraysUserBuf(struct gl_context *ctx,
+                                       const struct marshal_cmd_MultiDrawArraysUserBuf *cmd)
 {
    const GLenum mode = cmd->mode;
    const GLsizei draw_count = cmd->draw_count;
@@ -468,16 +503,16 @@ multi_draw_arrays_async(struct gl_context *ctx, GLenum mode,
    int first_size = sizeof(GLint) * draw_count;
    int count_size = sizeof(GLsizei) * draw_count;
    int buffers_size = util_bitcount(user_buffer_mask) * sizeof(buffers[0]);
-   int cmd_size = sizeof(struct marshal_cmd_MultiDrawArrays) +
+   int cmd_size = sizeof(struct marshal_cmd_MultiDrawArraysUserBuf) +
                   first_size + count_size + buffers_size;
-   struct marshal_cmd_MultiDrawArrays *cmd;
+   struct marshal_cmd_MultiDrawArraysUserBuf *cmd;
 
    /* Make sure cmd can fit the queue buffer */
    if (cmd_size > MARSHAL_MAX_CMD_SIZE) {
       return false;
    }
 
-   cmd = _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_MultiDrawArrays,
+   cmd = _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_MultiDrawArraysUserBuf,
                                          cmd_size);
    cmd->mode = mode;
    cmd->draw_count = draw_count;
@@ -515,7 +550,7 @@ _mesa_marshal_MultiDrawArrays(GLenum mode, const GLint *first,
    }
 
    /* If the draw count is too high or negative, the queue can't be used. */
-   if (!ctx->GLThread.SupportsNonVBOUploads ||
+   if (!ctx->GLThread.SupportsBufferUploads ||
        draw_count < 0 || draw_count > MARSHAL_MAX_CMD_SIZE / 16)
       goto sync;
 
@@ -560,14 +595,68 @@ sync:
                         (mode, first, count, draw_count));
 }
 
-/* DrawElementsInstancedBaseVertexBaseInstance not supporting user buffers.
- * Ignore the name.
- */
-struct marshal_cmd_DrawElementsInstancedARB
+/* DrawElementsInstanced without user buffers. */
+struct marshal_cmd_DrawElementsInstanced
 {
    struct marshal_cmd_base cmd_base;
-   GLenum mode;
-   GLenum type;
+   GLenum16 mode;
+   GLenum16 type;
+   GLsizei count;
+   GLsizei instance_count;
+   const GLvoid *indices;
+};
+
+uint32_t
+_mesa_unmarshal_DrawElementsInstanced(struct gl_context *ctx,
+                                      const struct marshal_cmd_DrawElementsInstanced *cmd)
+{
+   const GLenum mode = cmd->mode;
+   const GLsizei count = cmd->count;
+   const GLenum type = cmd->type;
+   const GLvoid *indices = cmd->indices;
+   const GLsizei instance_count = cmd->instance_count;
+
+   CALL_DrawElementsInstanced(ctx->CurrentServerDispatch,
+                              (mode, count, type, indices, instance_count));
+   const unsigned cmd_size = align(sizeof(*cmd), 8) / 8;
+   assert(cmd_size == cmd->cmd_base.cmd_size);
+   return cmd_size;
+}
+
+/* DrawElementsBaseVertex without user buffers. */
+struct marshal_cmd_DrawElementsBaseVertex
+{
+   struct marshal_cmd_base cmd_base;
+   GLenum16 mode;
+   GLenum16 type;
+   GLsizei count;
+   GLint basevertex;
+   const GLvoid *indices;
+};
+
+uint32_t
+_mesa_unmarshal_DrawElementsBaseVertex(struct gl_context *ctx,
+                                       const struct marshal_cmd_DrawElementsBaseVertex *cmd)
+{
+   const GLenum mode = cmd->mode;
+   const GLsizei count = cmd->count;
+   const GLenum type = cmd->type;
+   const GLvoid *indices = cmd->indices;
+   const GLint basevertex = cmd->basevertex;
+
+   CALL_DrawElementsBaseVertex(ctx->CurrentServerDispatch,
+                               (mode, count, type, indices, basevertex));
+   const unsigned cmd_size = align(sizeof(*cmd), 8) / 8;
+   assert(cmd_size == cmd->cmd_base.cmd_size);
+   return cmd_size;
+}
+
+/* DrawElementsInstancedBaseVertexBaseInstance without user buffers. */
+struct marshal_cmd_DrawElementsInstancedBaseVertexBaseInstance
+{
+   struct marshal_cmd_base cmd_base;
+   GLenum16 mode;
+   GLenum16 type;
    GLsizei count;
    GLsizei instance_count;
    GLint basevertex;
@@ -576,16 +665,9 @@ struct marshal_cmd_DrawElementsInstancedARB
 };
 
 uint32_t
-_mesa_unmarshal_DrawElementsInstancedARB(struct gl_context *ctx,
-                                         const struct marshal_cmd_DrawElementsInstancedARB *cmd,
-                                         const uint64_t *last)
+_mesa_unmarshal_DrawElementsInstancedBaseVertexBaseInstance(struct gl_context *ctx,
+                                                            const struct marshal_cmd_DrawElementsInstancedBaseVertexBaseInstance *cmd)
 {
-   /* Ignore the function name. We use DISPATCH_CMD_DrawElementsInstanced-
-    * BaseVertexBaseInstance for all DrawElements variants with user buffers,
-    * and both DISPATCH_CMD_DrawElementsInstancedARB and DISPATCH_CMD_Draw-
-    * RangeElementsBaseVertex for all draw elements variants without user
-    * buffers.
-    */
    const GLenum mode = cmd->mode;
    const GLsizei count = cmd->count;
    const GLenum type = cmd->type;
@@ -598,14 +680,16 @@ _mesa_unmarshal_DrawElementsInstancedARB(struct gl_context *ctx,
                                                     (mode, count, type, indices,
                                                      instance_count, basevertex,
                                                      baseinstance));
-   return cmd->cmd_base.cmd_size;
+   const unsigned cmd_size = align(sizeof(*cmd), 8) / 8;
+   assert(cmd_size == cmd->cmd_base.cmd_size);
+   return cmd_size;
 }
 
 struct marshal_cmd_DrawRangeElementsBaseVertex
 {
    struct marshal_cmd_base cmd_base;
-   GLenum mode;
-   GLenum type;
+   GLenum16 mode;
+   GLenum16 type;
    GLsizei count;
    GLint basevertex;
    GLuint min_index;
@@ -615,8 +699,7 @@ struct marshal_cmd_DrawRangeElementsBaseVertex
 
 uint32_t
 _mesa_unmarshal_DrawRangeElementsBaseVertex(struct gl_context *ctx,
-                                            const struct marshal_cmd_DrawRangeElementsBaseVertex *cmd,
-                                            const uint64_t *last)
+                                            const struct marshal_cmd_DrawRangeElementsBaseVertex *cmd)
 {
    const GLenum mode = cmd->mode;
    const GLsizei count = cmd->count;
@@ -629,7 +712,9 @@ _mesa_unmarshal_DrawRangeElementsBaseVertex(struct gl_context *ctx,
    CALL_DrawRangeElementsBaseVertex(ctx->CurrentServerDispatch,
                                     (mode, min_index, max_index, count,
                                      type, indices, basevertex));
-   return cmd->cmd_base.cmd_size;
+   const unsigned cmd_size = align(sizeof(*cmd), 8) / 8;
+   assert(cmd_size == cmd->cmd_base.cmd_size);
+   return cmd_size;
 }
 
 static ALWAYS_INLINE void
@@ -638,39 +723,63 @@ draw_elements_async(struct gl_context *ctx, GLenum mode, GLsizei count,
                     GLint basevertex, GLuint baseinstance,
                     bool index_bounds_valid, GLuint min_index, GLuint max_index)
 {
-   if (index_bounds_valid) {
-      int cmd_size = sizeof(struct marshal_cmd_DrawRangeElementsBaseVertex);
-      struct marshal_cmd_DrawRangeElementsBaseVertex *cmd =
-         _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawRangeElementsBaseVertex, cmd_size);
+   if (instance_count == 1 && baseinstance == 0) {
+      if (index_bounds_valid) {
+         int cmd_size = sizeof(struct marshal_cmd_DrawRangeElementsBaseVertex);
+         struct marshal_cmd_DrawRangeElementsBaseVertex *cmd =
+            _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawRangeElementsBaseVertex, cmd_size);
 
-      cmd->mode = mode;
-      cmd->count = count;
-      cmd->type = type;
-      cmd->indices = indices;
-      cmd->basevertex = basevertex;
-      cmd->min_index = min_index;
-      cmd->max_index = max_index;
+         cmd->mode = MIN2(mode, 0xffff);
+         cmd->type = MIN2(type, 0xffff);
+         cmd->count = count;
+         cmd->indices = indices;
+         cmd->basevertex = basevertex;
+         cmd->min_index = min_index;
+         cmd->max_index = max_index;
+      } else {
+         int cmd_size = sizeof(struct marshal_cmd_DrawElementsBaseVertex);
+         struct marshal_cmd_DrawElementsBaseVertex *cmd =
+            _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawElementsBaseVertex, cmd_size);
+
+         cmd->mode = MIN2(mode, 0xffff);
+         cmd->type = MIN2(type, 0xffff);
+         cmd->count = count;
+         cmd->indices = indices;
+         cmd->basevertex = basevertex;
+      }
    } else {
-      int cmd_size = sizeof(struct marshal_cmd_DrawElementsInstancedARB);
-      struct marshal_cmd_DrawElementsInstancedARB *cmd =
-         _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawElementsInstancedARB, cmd_size);
+      if (basevertex == 0 && baseinstance == 0) {
+         int cmd_size = sizeof(struct marshal_cmd_DrawElementsInstanced);
+         struct marshal_cmd_DrawElementsInstanced *cmd =
+            _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawElementsInstanced, cmd_size);
 
-      cmd->mode = mode;
-      cmd->count = count;
-      cmd->type = type;
-      cmd->indices = indices;
-      cmd->instance_count = instance_count;
-      cmd->basevertex = basevertex;
-      cmd->baseinstance = baseinstance;
+         cmd->mode = MIN2(mode, 0xffff);
+         cmd->type = MIN2(type, 0xffff);
+         cmd->count = count;
+         cmd->instance_count = instance_count;
+         cmd->indices = indices;
+      } else {
+         int cmd_size = sizeof(struct marshal_cmd_DrawElementsInstancedBaseVertexBaseInstance);
+         struct marshal_cmd_DrawElementsInstancedBaseVertexBaseInstance *cmd =
+            _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawElementsInstancedBaseVertexBaseInstance, cmd_size);
+
+         cmd->mode = MIN2(mode, 0xffff);
+         cmd->type = MIN2(type, 0xffff);
+         cmd->count = count;
+         cmd->instance_count = instance_count;
+         cmd->basevertex = basevertex;
+         cmd->baseinstance = baseinstance;
+         cmd->indices = indices;
+      }
    }
 }
 
-struct marshal_cmd_DrawElementsInstancedBaseVertexBaseInstance
+struct marshal_cmd_DrawElementsUserBuf
 {
    struct marshal_cmd_base cmd_base;
    bool index_bounds_valid;
-   GLenum mode;
-   GLenum type;
+   GLenum8 mode;
+   GLenum16 type;
    GLsizei count;
    GLsizei instance_count;
    GLint basevertex;
@@ -683,16 +792,9 @@ struct marshal_cmd_DrawElementsInstancedBaseVertexBaseInstance
 };
 
 uint32_t
-_mesa_unmarshal_DrawElementsInstancedBaseVertexBaseInstance(struct gl_context *ctx,
-                                                            const struct marshal_cmd_DrawElementsInstancedBaseVertexBaseInstance *cmd,
-                                                            const uint64_t *last)
+_mesa_unmarshal_DrawElementsUserBuf(struct gl_context *ctx,
+                                    const struct marshal_cmd_DrawElementsUserBuf *cmd)
 {
-   /* Ignore the function name. We use DISPATCH_CMD_DrawElementsInstanced-
-    * BaseVertexBaseInstance for all DrawElements variants with user buffers,
-    * and both DISPATCH_CMD_DrawElementsInstancedARB and DISPATCH_CMD_Draw-
-    * RangeElementsBaseVertex for all draw elements variants without user
-    * buffers.
-    */
    const GLenum mode = cmd->mode;
    const GLsizei count = cmd->count;
    const GLenum type = cmd->type;
@@ -749,14 +851,14 @@ draw_elements_async_user(struct gl_context *ctx, GLenum mode, GLsizei count,
                          const struct glthread_attrib_binding *buffers)
 {
    int buffers_size = util_bitcount(user_buffer_mask) * sizeof(buffers[0]);
-   int cmd_size = sizeof(struct marshal_cmd_DrawElementsInstancedBaseVertexBaseInstance) +
+   int cmd_size = sizeof(struct marshal_cmd_DrawElementsUserBuf) +
                   buffers_size;
-   struct marshal_cmd_DrawElementsInstancedBaseVertexBaseInstance *cmd;
+   struct marshal_cmd_DrawElementsUserBuf *cmd;
 
-   cmd = _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawElementsInstancedBaseVertexBaseInstance, cmd_size);
-   cmd->mode = mode;
+   cmd = _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_DrawElementsUserBuf, cmd_size);
+   cmd->mode = MIN2(mode, 0xff); /* primitive types go from 0 to 14 */
+   cmd->type = MIN2(type, 0xffff);
    cmd->count = count;
-   cmd->type = type;
    cmd->indices = indices;
    cmd->instance_count = instance_count;
    cmd->basevertex = basevertex;
@@ -794,14 +896,16 @@ draw_elements(GLenum mode, GLsizei count, GLenum type, const GLvoid *indices,
    if (ctx->API == API_OPENGL_CORE ||
        count <= 0 || instance_count <= 0 || max_index < min_index ||
        !is_index_type_valid(type) ||
-       (!user_buffer_mask && !has_user_indices)) {
+       (!user_buffer_mask && !has_user_indices) ||
+       /* This will just generate GL_INVALID_OPERATION, as it should. */
+       (!compiled_into_dlist && ctx->GLThread.ListMode)) {
       draw_elements_async(ctx, mode, count, type, indices, instance_count,
                           basevertex, baseinstance, index_bounds_valid,
                           min_index, max_index);
       return;
    }
 
-   if (!ctx->GLThread.SupportsNonVBOUploads)
+   if (!ctx->GLThread.SupportsBufferUploads)
       goto sync;
 
    bool need_index_bounds = user_buffer_mask & ~vao->NonZeroDivisorMask;
@@ -879,21 +983,20 @@ sync:
    }
 }
 
-struct marshal_cmd_MultiDrawElementsBaseVertex
+struct marshal_cmd_MultiDrawElementsUserBuf
 {
    struct marshal_cmd_base cmd_base;
    bool has_base_vertex;
-   GLenum mode;
-   GLenum type;
+   GLenum8 mode;
+   GLenum16 type;
    GLsizei draw_count;
    GLuint user_buffer_mask;
    struct gl_buffer_object *index_buffer;
 };
 
 uint32_t
-_mesa_unmarshal_MultiDrawElementsBaseVertex(struct gl_context *ctx,
-                                            const struct marshal_cmd_MultiDrawElementsBaseVertex *cmd,
-                                            const uint64_t *last)
+_mesa_unmarshal_MultiDrawElementsUserBuf(struct gl_context *ctx,
+                                         const struct marshal_cmd_MultiDrawElementsUserBuf *cmd)
 {
    const GLenum mode = cmd->mode;
    const GLenum type = cmd->type;
@@ -930,8 +1033,8 @@ _mesa_unmarshal_MultiDrawElementsBaseVertex(struct gl_context *ctx,
                                        (mode, count, type, indices, draw_count,
                                         basevertex));
    } else {
-      CALL_MultiDrawElementsEXT(ctx->CurrentServerDispatch,
-                                (mode, count, type, indices, draw_count));
+      CALL_MultiDrawElements(ctx->CurrentServerDispatch,
+                             (mode, count, type, indices, draw_count));
    }
 
    /* Restore states. */
@@ -958,18 +1061,18 @@ multi_draw_elements_async(struct gl_context *ctx, GLenum mode,
    int indices_size = sizeof(indices[0]) * draw_count;
    int basevertex_size = basevertex ? sizeof(GLsizei) * draw_count : 0;
    int buffers_size = util_bitcount(user_buffer_mask) * sizeof(buffers[0]);
-   int cmd_size = sizeof(struct marshal_cmd_MultiDrawElementsBaseVertex) +
+   int cmd_size = sizeof(struct marshal_cmd_MultiDrawElementsUserBuf) +
                   count_size + indices_size + basevertex_size + buffers_size;
-   struct marshal_cmd_MultiDrawElementsBaseVertex *cmd;
+   struct marshal_cmd_MultiDrawElementsUserBuf *cmd;
 
    /* Make sure cmd can fit the queue buffer */
    if (cmd_size > MARSHAL_MAX_CMD_SIZE) {
       return false;
    }
 
-   cmd = _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_MultiDrawElementsBaseVertex, cmd_size);
-   cmd->mode = mode;
-   cmd->type = type;
+   cmd = _mesa_glthread_allocate_command(ctx, DISPATCH_CMD_MultiDrawElementsUserBuf, cmd_size);
+   cmd->mode = MIN2(mode, 0xff); /* primitive types go from 0 to 14 */
+   cmd->type = MIN2(type, 0xffff);
    cmd->draw_count = draw_count;
    cmd->user_buffer_mask = user_buffer_mask;
    cmd->index_buffer = index_buffer;
@@ -1026,7 +1129,7 @@ _mesa_marshal_MultiDrawElementsBaseVertex(GLenum mode, const GLsizei *count,
     * and index bounds are not valid. We would have to map the indices
     * to compute the index bounds, and for that we would have to sync anyway.
     */
-   if (!ctx->GLThread.SupportsNonVBOUploads ||
+   if (!ctx->GLThread.SupportsBufferUploads ||
        draw_count < 0 || draw_count > MARSHAL_MAX_CMD_SIZE / 32 ||
        (need_index_bounds && !has_user_indices))
       goto sync;
@@ -1138,8 +1241,8 @@ sync:
                                        (mode, count, type, indices, draw_count,
                                         basevertex));
    } else {
-      CALL_MultiDrawElementsEXT(ctx->CurrentServerDispatch,
-                                (mode, count, type, indices, draw_count));
+      CALL_MultiDrawElements(ctx->CurrentServerDispatch,
+                             (mode, count, type, indices, draw_count));
    }
 }
 
@@ -1150,8 +1253,8 @@ _mesa_marshal_DrawArrays(GLenum mode, GLint first, GLsizei count)
 }
 
 void GLAPIENTRY
-_mesa_marshal_DrawArraysInstancedARB(GLenum mode, GLint first, GLsizei count,
-                                     GLsizei instance_count)
+_mesa_marshal_DrawArraysInstanced(GLenum mode, GLint first, GLsizei count,
+                                  GLsizei instance_count)
 {
    draw_arrays(mode, first, count, instance_count, 0, false);
 }
@@ -1180,8 +1283,8 @@ _mesa_marshal_DrawRangeElements(GLenum mode, GLuint start, GLuint end,
 }
 
 void GLAPIENTRY
-_mesa_marshal_DrawElementsInstancedARB(GLenum mode, GLsizei count, GLenum type,
-                                       const GLvoid *indices, GLsizei instance_count)
+_mesa_marshal_DrawElementsInstanced(GLenum mode, GLsizei count, GLenum type,
+                                    const GLvoid *indices, GLsizei instance_count)
 {
    draw_elements(mode, count, type, indices, instance_count, 0, 0, false, 0, 0, false);
 }
@@ -1198,7 +1301,7 @@ _mesa_marshal_DrawRangeElementsBaseVertex(GLenum mode, GLuint start, GLuint end,
                                           GLsizei count, GLenum type,
                                           const GLvoid *indices, GLint basevertex)
 {
-   draw_elements(mode, count, type, indices, 1, basevertex, 0, true, start, end, false);
+   draw_elements(mode, count, type, indices, 1, basevertex, 0, true, start, end, true);
 }
 
 void GLAPIENTRY
@@ -1227,59 +1330,122 @@ _mesa_marshal_DrawElementsInstancedBaseVertexBaseInstance(GLenum mode, GLsizei c
 }
 
 void GLAPIENTRY
-_mesa_marshal_MultiDrawElementsEXT(GLenum mode, const GLsizei *count,
-                                   GLenum type, const GLvoid *const *indices,
-                                   GLsizei draw_count)
+_mesa_marshal_MultiDrawElements(GLenum mode, const GLsizei *count,
+                                GLenum type, const GLvoid *const *indices,
+                                GLsizei draw_count)
 {
    _mesa_marshal_MultiDrawElementsBaseVertex(mode, count, type, indices,
                                              draw_count, NULL);
 }
 
 uint32_t
-_mesa_unmarshal_DrawArraysInstancedARB(struct gl_context *ctx, const struct marshal_cmd_DrawArraysInstancedARB *cmd, const uint64_t *last)
+_mesa_unmarshal_DrawArraysInstanced(struct gl_context *ctx,
+                                    const struct marshal_cmd_DrawArraysInstanced *cmd)
 {
-   unreachable("never used - DrawArraysInstancedBaseInstance is used instead");
+   unreachable("should never end up here");
    return 0;
 }
 
 uint32_t
-_mesa_unmarshal_DrawElements(struct gl_context *ctx, const struct marshal_cmd_DrawElements *cmd, const uint64_t *last)
+_mesa_unmarshal_MultiDrawArrays(struct gl_context *ctx,
+                                const struct marshal_cmd_MultiDrawArrays *cmd)
 {
-   unreachable("never used - DrawElementsInstancedBaseVertexBaseInstance is used instead");
+   unreachable("should never end up here");
    return 0;
 }
 
 uint32_t
-_mesa_unmarshal_DrawRangeElements(struct gl_context *ctx, const struct marshal_cmd_DrawRangeElements *cmd, const uint64_t *last)
+_mesa_unmarshal_DrawElements(struct gl_context *ctx,
+                             const struct marshal_cmd_DrawElements *cmd)
 {
-   unreachable("never used - DrawElementsInstancedBaseVertexBaseInstance is used instead");
+   unreachable("should never end up here");
    return 0;
 }
 
 uint32_t
-_mesa_unmarshal_DrawElementsBaseVertex(struct gl_context *ctx, const struct marshal_cmd_DrawElementsBaseVertex *cmd, const uint64_t *last)
+_mesa_unmarshal_DrawRangeElements(struct gl_context *ctx,
+                                  const struct marshal_cmd_DrawRangeElements *cmd)
 {
-   unreachable("never used - DrawElementsInstancedBaseVertexBaseInstance is used instead");
+   unreachable("should never end up here");
    return 0;
 }
 
 uint32_t
-_mesa_unmarshal_DrawElementsInstancedBaseVertex(struct gl_context *ctx, const struct marshal_cmd_DrawElementsInstancedBaseVertex *cmd, const uint64_t *last)
+_mesa_unmarshal_DrawElementsInstancedBaseVertex(struct gl_context *ctx,
+                                                const struct marshal_cmd_DrawElementsInstancedBaseVertex *cmd)
 {
-   unreachable("never used - DrawElementsInstancedBaseVertexBaseInstance is used instead");
+   unreachable("should never end up here");
    return 0;
 }
 
 uint32_t
-_mesa_unmarshal_DrawElementsInstancedBaseInstance(struct gl_context *ctx, const struct marshal_cmd_DrawElementsInstancedBaseInstance *cmd, const uint64_t *last)
+_mesa_unmarshal_DrawElementsInstancedBaseInstance(struct gl_context *ctx,
+                                                  const struct marshal_cmd_DrawElementsInstancedBaseInstance *cmd)
 {
-   unreachable("never used - DrawElementsInstancedBaseVertexBaseInstance is used instead");
+   unreachable("should never end up here");
    return 0;
 }
 
 uint32_t
-_mesa_unmarshal_MultiDrawElementsEXT(struct gl_context *ctx, const struct marshal_cmd_MultiDrawElementsEXT *cmd, const uint64_t *last)
+_mesa_unmarshal_MultiDrawElements(struct gl_context *ctx,
+                                  const struct marshal_cmd_MultiDrawElements *cmd)
 {
-   unreachable("never used - MultiDrawElementsBaseVertex is used instead");
+   unreachable("should never end up here");
    return 0;
 }
+
+uint32_t
+_mesa_unmarshal_MultiDrawElementsBaseVertex(struct gl_context *ctx,
+                                            const struct marshal_cmd_MultiDrawElementsBaseVertex *cmd)
+{
+   unreachable("should never end up here");
+   return 0;
+}
+
+void GLAPIENTRY
+_mesa_marshal_DrawArraysUserBuf(void)
+{
+   unreachable("should never end up here");
+}
+
+void GLAPIENTRY
+_mesa_marshal_DrawElementsUserBuf(void)
+{
+   unreachable("should never end up here");
+}
+
+void GLAPIENTRY
+_mesa_marshal_MultiDrawArraysUserBuf(void)
+{
+   unreachable("should never end up here");
+}
+
+void GLAPIENTRY
+_mesa_marshal_MultiDrawElementsUserBuf(void)
+{
+   unreachable("should never end up here");
+}
+
+void GLAPIENTRY
+_mesa_DrawArraysUserBuf(void)
+{
+   unreachable("should never end up here");
+}
+
+void GLAPIENTRY
+_mesa_DrawElementsUserBuf(void)
+{
+   unreachable("should never end up here");
+}
+
+void GLAPIENTRY
+_mesa_MultiDrawArraysUserBuf(void)
+{
+   unreachable("should never end up here");
+}
+
+void GLAPIENTRY
+_mesa_MultiDrawElementsUserBuf(void)
+{
+   unreachable("should never end up here");
+}
