@@ -34,7 +34,6 @@ USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "main/bufferobj.h"
 #include "main/context.h"
 #include "main/macros.h"
-#include "main/vtxfmt.h"
 #include "main/dlist.h"
 #include "main/eval.h"
 #include "main/state.h"
@@ -44,25 +43,14 @@ USE OR OTHER DEALINGS IN THE SOFTWARE.
 #include "main/dispatch.h"
 #include "util/bitscan.h"
 #include "util/u_memory.h"
+#include "api_exec_decl.h"
 
-#include "vbo_noop.h"
 #include "vbo_private.h"
 
-
 /** ID/name for immediate-mode VBO */
 #define IMM_BUFFER_NAME 0xaabbccdd
 
 
-static void GLAPIENTRY
-vbo_exec_Materialfv(GLenum face, GLenum pname, const GLfloat *params);
-
-static void GLAPIENTRY
-vbo_exec_EvalCoord1f(GLfloat u);
-
-static void GLAPIENTRY
-vbo_exec_EvalCoord2f(GLfloat u, GLfloat v);
-
-
 static void
 vbo_reset_all_attr(struct vbo_exec_context *exec);
 
@@ -83,7 +71,7 @@ vbo_exec_wrap_buffers(struct vbo_exec_context *exec)
    else {
       struct gl_context *ctx = gl_context_from_vbo_exec(exec);
       unsigned last = exec->vtx.prim_count - 1;
-      struct pipe_draw_start_count *last_draw = &exec->vtx.draw[last];
+      struct pipe_draw_start_count_bias *last_draw = &exec->vtx.draw[last];
       const bool last_begin = exec->vtx.markers[last].begin;
       GLuint last_count = 0;
 
@@ -192,6 +180,10 @@ vbo_exec_copy_to_current(struct vbo_exec_context *exec)
 
       assert(exec->vtx.attr[i].size);
 
+      /* VBO_ATTRIB_SELECT_RESULT_INDEX has no current */
+      if (!current)
+         continue;
+
       if (exec->vtx.attr[i].type == GL_DOUBLE ||
           exec->vtx.attr[i].type == GL_UNSIGNED_INT64_ARB) {
          memset(tmp, 0, sizeof(tmp));
@@ -268,6 +260,13 @@ vbo_exec_wrap_upgrade_vertex(struct vbo_exec_context *exec,
 
    assert(attr < VBO_ATTRIB_MAX);
 
+   if (unlikely(!exec->vtx.buffer_ptr)) {
+      /* We should only hit this when use_buffer_objects=true */
+      assert(exec->vtx.bufferobj);
+      vbo_exec_vtx_map(exec);
+      assert(exec->vtx.buffer_ptr);
+   }
+
    /* Run pipeline on current vertices, copy wrapped vertices
     * to exec->vtx.copied.
     */
@@ -482,13 +481,12 @@ is_vertex_position(const struct gl_context *ctx, GLuint index)
  * \param C  cast type (uint32_t or uint64_t)
  * \param V0, V1, v2, V3  attribute value
  */
-#define ATTR_UNION(A, N, T, C, V0, V1, V2, V3)                          \
+#define ATTR_UNION_BASE(A, N, T, C, V0, V1, V2, V3)                     \
 do {                                                                    \
    struct vbo_exec_context *exec = &vbo_context(ctx)->exec;             \
    int sz = (sizeof(C) / sizeof(GLfloat));                              \
                                                                         \
    assert(sz == 1 || sz == 2);                                          \
-                                                                        \
    /* store a copy of the attribute in exec except for glVertex */      \
    if ((A) != 0) {                                                      \
       /* Check if attribute size or type is changing. */                \
@@ -563,21 +561,23 @@ do {                                                                    \
    }                                                                    \
 } while (0)
 
-
 #undef ERROR
 #define ERROR(err) _mesa_error(ctx, err, __func__)
-#define TAG(x) vbo_exec_##x
+#define TAG(x) _mesa_##x
+#define SUPPRESS_STATIC
+
+#define ATTR_UNION(A, N, T, C, V0, V1, V2, V3) \
+   ATTR_UNION_BASE(A, N, T, C, V0, V1, V2, V3)
 
 #include "vbo_attrib_tmp.h"
 
 
-
 /**
  * Execute a glMaterial call.  Note that if GL_COLOR_MATERIAL is enabled,
  * this may be a (partial) no-op.
  */
-static void GLAPIENTRY
-vbo_exec_Materialfv(GLenum face, GLenum pname, const GLfloat *params)
+void GLAPIENTRY
+_mesa_Materialfv(GLenum face, GLenum pname, const GLfloat *params)
 {
    GLbitfield updateMats;
    GET_CURRENT_CONTEXT(ctx);
@@ -706,8 +706,8 @@ vbo_exec_FlushVertices_internal(struct vbo_exec_context *exec, unsigned flags)
 }
 
 
-static void GLAPIENTRY
-vbo_exec_EvalCoord1f(GLfloat u)
+void GLAPIENTRY
+_mesa_EvalCoord1f(GLfloat u)
 {
    GET_CURRENT_CONTEXT(ctx);
    struct vbo_exec_context *exec = &vbo_context(ctx)->exec;
@@ -734,8 +734,8 @@ vbo_exec_EvalCoord1f(GLfloat u)
 }
 
 
-static void GLAPIENTRY
-vbo_exec_EvalCoord2f(GLfloat u, GLfloat v)
+void GLAPIENTRY
+_mesa_EvalCoord2f(GLfloat u, GLfloat v)
 {
    GET_CURRENT_CONTEXT(ctx);
    struct vbo_exec_context *exec = &vbo_context(ctx)->exec;
@@ -766,34 +766,34 @@ vbo_exec_EvalCoord2f(GLfloat u, GLfloat v)
 }
 
 
-static void GLAPIENTRY
-vbo_exec_EvalCoord1fv(const GLfloat *u)
+void GLAPIENTRY
+_mesa_EvalCoord1fv(const GLfloat *u)
 {
-   vbo_exec_EvalCoord1f(u[0]);
+   _mesa_EvalCoord1f(u[0]);
 }
 
 
-static void GLAPIENTRY
-vbo_exec_EvalCoord2fv(const GLfloat *u)
+void GLAPIENTRY
+_mesa_EvalCoord2fv(const GLfloat *u)
 {
-   vbo_exec_EvalCoord2f(u[0], u[1]);
+   _mesa_EvalCoord2f(u[0], u[1]);
 }
 
 
-static void GLAPIENTRY
-vbo_exec_EvalPoint1(GLint i)
+void GLAPIENTRY
+_mesa_EvalPoint1(GLint i)
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat du = ((ctx->Eval.MapGrid1u2 - ctx->Eval.MapGrid1u1) /
                  (GLfloat) ctx->Eval.MapGrid1un);
    GLfloat u = i * du + ctx->Eval.MapGrid1u1;
 
-   vbo_exec_EvalCoord1f(u);
+   _mesa_EvalCoord1f(u);
 }
 
 
-static void GLAPIENTRY
-vbo_exec_EvalPoint2(GLint i, GLint j)
+void GLAPIENTRY
+_mesa_EvalPoint2(GLint i, GLint j)
 {
    GET_CURRENT_CONTEXT(ctx);
    GLfloat du = ((ctx->Eval.MapGrid2u2 - ctx->Eval.MapGrid2u1) /
@@ -803,15 +803,15 @@ vbo_exec_EvalPoint2(GLint i, GLint j)
    GLfloat u = i * du + ctx->Eval.MapGrid2u1;
    GLfloat v = j * dv + ctx->Eval.MapGrid2v1;
 
-   vbo_exec_EvalCoord2f(u, v);
+   _mesa_EvalCoord2f(u, v);
 }
 
 
 /**
  * Called via glBegin.
  */
-static void GLAPIENTRY
-vbo_exec_Begin(GLenum mode)
+void GLAPIENTRY
+_mesa_Begin(GLenum mode)
 {
    GET_CURRENT_CONTEXT(ctx);
    struct vbo_context *vbo = vbo_context(ctx);
@@ -848,15 +848,17 @@ vbo_exec_Begin(GLenum mode)
 
    ctx->Driver.CurrentExecPrimitive = mode;
 
-   ctx->Exec = ctx->BeginEnd;
+   ctx->Exec = _mesa_hw_select_enabled(ctx) ?
+      ctx->HWSelectModeBeginEnd : ctx->BeginEnd;
 
    /* We may have been called from a display list, in which case we should
     * leave dlist.c's dispatch table in place.
     */
-   if (ctx->CurrentClientDispatch == ctx->MarshalExec) {
-      ctx->CurrentServerDispatch = ctx->Exec;
+   if (ctx->GLThread.enabled) {
+      if (ctx->CurrentServerDispatch == ctx->OutsideBeginEnd)
+         ctx->CurrentServerDispatch = ctx->Exec;
    } else if (ctx->CurrentClientDispatch == ctx->OutsideBeginEnd) {
-      ctx->CurrentClientDispatch = ctx->Exec;
+      ctx->CurrentClientDispatch = ctx->CurrentServerDispatch = ctx->Exec;
       _glapi_set_dispatch(ctx->CurrentClientDispatch);
    } else {
       assert(ctx->CurrentClientDispatch == ctx->Save);
@@ -899,8 +901,8 @@ try_vbo_merge(struct vbo_exec_context *exec)
 /**
  * Called via glEnd.
  */
-static void GLAPIENTRY
-vbo_exec_End(void)
+void GLAPIENTRY
+_mesa_End(void)
 {
    GET_CURRENT_CONTEXT(ctx);
    struct vbo_exec_context *exec = &vbo_context(ctx)->exec;
@@ -912,24 +914,33 @@ vbo_exec_End(void)
 
    ctx->Exec = ctx->OutsideBeginEnd;
 
-   if (ctx->CurrentClientDispatch == ctx->MarshalExec) {
-      ctx->CurrentServerDispatch = ctx->Exec;
-   } else if (ctx->CurrentClientDispatch == ctx->BeginEnd) {
-      ctx->CurrentClientDispatch = ctx->Exec;
+   if (ctx->GLThread.enabled) {
+      if (ctx->CurrentServerDispatch == ctx->BeginEnd ||
+          ctx->CurrentServerDispatch == ctx->HWSelectModeBeginEnd) {
+         ctx->CurrentServerDispatch = ctx->Exec;
+      }
+   } else if (ctx->CurrentClientDispatch == ctx->BeginEnd ||
+              ctx->CurrentClientDispatch == ctx->HWSelectModeBeginEnd) {
+      ctx->CurrentClientDispatch = ctx->CurrentServerDispatch = ctx->Exec;
       _glapi_set_dispatch(ctx->CurrentClientDispatch);
    }
 
    if (exec->vtx.prim_count > 0) {
       /* close off current primitive */
       unsigned last = exec->vtx.prim_count - 1;
-      struct pipe_draw_start_count *last_draw = &exec->vtx.draw[last];
+      struct pipe_draw_start_count_bias *last_draw = &exec->vtx.draw[last];
       unsigned count = exec->vtx.vert_count - last_draw->start;
 
       last_draw->count = count;
       exec->vtx.markers[last].end = 1;
 
-      if (count)
+      if (count) {
+         /* mark result buffer used */
+         if (_mesa_hw_select_enabled(ctx))
+            ctx->Select.ResultUsed = GL_TRUE;
+
          ctx->Driver.NeedFlush |= FLUSH_STORED_VERTICES;
+      }
 
       /* Special handling for GL_LINE_LOOP */
       if (exec->vtx.mode[last] == GL_LINE_LOOP &&
@@ -973,8 +984,8 @@ vbo_exec_End(void)
 /**
  * Called via glPrimitiveRestartNV()
  */
-static void GLAPIENTRY
-vbo_exec_PrimitiveRestartNV(void)
+void GLAPIENTRY
+_mesa_PrimitiveRestartNV(void)
 {
    GLenum curPrim;
    GET_CURRENT_CONTEXT(ctx);
@@ -985,24 +996,97 @@ vbo_exec_PrimitiveRestartNV(void)
       _mesa_error(ctx, GL_INVALID_OPERATION, "glPrimitiveRestartNV");
    }
    else {
-      vbo_exec_End();
-      vbo_exec_Begin(curPrim);
+      _mesa_End();
+      _mesa_Begin(curPrim);
    }
 }
 
 
+/**
+ * A special version of glVertexAttrib4f that does not treat index 0 as
+ * VBO_ATTRIB_POS.
+ */
 static void
-vbo_exec_vtxfmt_init(struct vbo_exec_context *exec)
+VertexAttrib4f_nopos(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
 {
-   struct gl_context *ctx = gl_context_from_vbo_exec(exec);
-   GLvertexformat *vfmt = &exec->vtxfmt;
+   GET_CURRENT_CONTEXT(ctx);
+   if (index < ctx->Const.Program[MESA_SHADER_VERTEX].MaxAttribs)
+      ATTRF(VBO_ATTRIB_GENERIC0 + index, 4, x, y, z, w);
+   else
+      ERROR(GL_INVALID_VALUE);
+}
 
-#define NAME_AE(x) _ae_##x
+static void GLAPIENTRY
+_es_VertexAttrib4fARB(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
+{
+   VertexAttrib4f_nopos(index, x, y, z, w);
+}
+
+
+static void GLAPIENTRY
+_es_VertexAttrib1fARB(GLuint indx, GLfloat x)
+{
+   VertexAttrib4f_nopos(indx, x, 0.0f, 0.0f, 1.0f);
+}
+
+
+static void GLAPIENTRY
+_es_VertexAttrib1fvARB(GLuint indx, const GLfloat* values)
+{
+   VertexAttrib4f_nopos(indx, values[0], 0.0f, 0.0f, 1.0f);
+}
+
+
+static void GLAPIENTRY
+_es_VertexAttrib2fARB(GLuint indx, GLfloat x, GLfloat y)
+{
+   VertexAttrib4f_nopos(indx, x, y, 0.0f, 1.0f);
+}
+
+
+static void GLAPIENTRY
+_es_VertexAttrib2fvARB(GLuint indx, const GLfloat* values)
+{
+   VertexAttrib4f_nopos(indx, values[0], values[1], 0.0f, 1.0f);
+}
+
+
+static void GLAPIENTRY
+_es_VertexAttrib3fARB(GLuint indx, GLfloat x, GLfloat y, GLfloat z)
+{
+   VertexAttrib4f_nopos(indx, x, y, z, 1.0f);
+}
+
+
+static void GLAPIENTRY
+_es_VertexAttrib3fvARB(GLuint indx, const GLfloat* values)
+{
+   VertexAttrib4f_nopos(indx, values[0], values[1], values[2], 1.0f);
+}
+
+
+static void GLAPIENTRY
+_es_VertexAttrib4fvARB(GLuint indx, const GLfloat* values)
+{
+   VertexAttrib4f_nopos(indx, values[0], values[1], values[2], values[3]);
+}
+
+
+void
+vbo_init_dispatch_begin_end(struct gl_context *ctx)
+{
+#define NAME_AE(x) _mesa_##x
 #define NAME_CALLLIST(x) _mesa_##x
-#define NAME(x) vbo_exec_##x
+#define NAME(x) _mesa_##x
 #define NAME_ES(x) _es_##x
 
-#include "vbo_init_tmp.h"
+   struct _glapi_table *tab = ctx->OutsideBeginEnd;
+   #include "api_beginend_init.h"
+
+   if (ctx->BeginEnd) {
+      tab = ctx->BeginEnd;
+      #include "api_beginend_init.h"
+   }
 }
 
 
@@ -1024,29 +1108,11 @@ vbo_reset_all_attr(struct vbo_exec_context *exec)
 
 
 void
-vbo_exec_vtx_init(struct vbo_exec_context *exec, bool use_buffer_objects)
+vbo_exec_vtx_init(struct vbo_exec_context *exec)
 {
    struct gl_context *ctx = gl_context_from_vbo_exec(exec);
 
-   if (use_buffer_objects) {
-      /* Use buffer objects for immediate mode. */
-      struct vbo_exec_context *exec = &vbo_context(ctx)->exec;
-
-      exec->vtx.bufferobj = ctx->Driver.NewBufferObject(ctx, IMM_BUFFER_NAME);
-
-      /* Map the buffer. */
-      vbo_exec_vtx_map(exec);
-      assert(exec->vtx.buffer_ptr);
-   } else {
-      /* Use allocated memory for immediate mode. */
-      exec->vtx.bufferobj = NULL;
-      exec->vtx.buffer_map =
-         align_malloc(ctx->Const.glBeginEndBufferSize, 64);
-      exec->vtx.buffer_ptr = exec->vtx.buffer_map;
-   }
-
-   vbo_exec_vtxfmt_init(exec);
-   _mesa_noop_vtxfmt_init(ctx, &exec->vtxfmt_noop);
+   exec->vtx.bufferobj = _mesa_bufferobj_alloc(ctx, IMM_BUFFER_NAME);
 
    exec->vtx.enabled = u_bit_consecutive64(0, VBO_ATTRIB_MAX); /* reset all */
    vbo_reset_all_attr(exec);
@@ -1078,7 +1144,7 @@ vbo_exec_vtx_destroy(struct vbo_exec_context *exec)
     */
    if (exec->vtx.bufferobj &&
        _mesa_bufferobj_mapped(exec->vtx.bufferobj, MAP_INTERNAL)) {
-      ctx->Driver.UnmapBuffer(ctx, exec->vtx.bufferobj, MAP_INTERNAL);
+      _mesa_bufferobj_unmap(ctx, exec->vtx.bufferobj, MAP_INTERNAL);
    }
    _mesa_reference_buffer_object(ctx, &exec->vtx.bufferobj, NULL);
 }
@@ -1128,28 +1194,28 @@ vbo_exec_FlushVertices(struct gl_context *ctx, GLuint flags)
 void GLAPIENTRY
 _es_Color4f(GLfloat r, GLfloat g, GLfloat b, GLfloat a)
 {
-   vbo_exec_Color4f(r, g, b, a);
+   _mesa_Color4f(r, g, b, a);
 }
 
 
 void GLAPIENTRY
 _es_Normal3f(GLfloat x, GLfloat y, GLfloat z)
 {
-   vbo_exec_Normal3f(x, y, z);
+   _mesa_Normal3f(x, y, z);
 }
 
 
 void GLAPIENTRY
 _es_MultiTexCoord4f(GLenum target, GLfloat s, GLfloat t, GLfloat r, GLfloat q)
 {
-   vbo_exec_MultiTexCoord4f(target, s, t, r, q);
+   _mesa_MultiTexCoord4fARB(target, s, t, r, q);
 }
 
 
 void GLAPIENTRY
 _es_Materialfv(GLenum face, GLenum pname, const GLfloat *params)
 {
-   vbo_exec_Materialfv(face, pname, params);
+   _mesa_Materialfv(face, pname, params);
 }
 
 
@@ -1159,75 +1225,35 @@ _es_Materialf(GLenum face, GLenum pname, GLfloat param)
    GLfloat p[4];
    p[0] = param;
    p[1] = p[2] = p[3] = 0.0F;
-   vbo_exec_Materialfv(face, pname, p);
+   _mesa_Materialfv(face, pname, p);
 }
 
+#undef TAG
+#undef SUPPRESS_STATIC
+#define TAG(x) _hw_select_##x
+/* filter out none vertex api */
+#define HW_SELECT_MODE
 
-/**
- * A special version of glVertexAttrib4f that does not treat index 0 as
- * VBO_ATTRIB_POS.
- */
-static void
-VertexAttrib4f_nopos(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
+#undef ATTR_UNION
+#define ATTR_UNION(A, N, T, C, V0, V1, V2, V3)     \
+   do {                                            \
+      if ((A) == 0) {                              \
+         ATTR_UNION_BASE(VBO_ATTRIB_SELECT_RESULT_OFFSET, 1, GL_UNSIGNED_INT, uint32_t, \
+                         ctx->Select.ResultOffset, 0, 0, 0); \
+      }                                            \
+      ATTR_UNION_BASE(A, N, T, C, V0, V1, V2, V3); \
+   } while (0)
+
+#include "vbo_attrib_tmp.h"
+
+void
+vbo_init_dispatch_hw_select_begin_end(struct gl_context *ctx)
 {
-   GET_CURRENT_CONTEXT(ctx);
-   if (index < MAX_VERTEX_GENERIC_ATTRIBS)
-      ATTRF(VBO_ATTRIB_GENERIC0 + index, 4, x, y, z, w);
-   else
-      ERROR(GL_INVALID_VALUE);
-}
+   int numEntries = MAX2(_gloffset_COUNT, _glapi_get_dispatch_table_size());
+   memcpy(ctx->HWSelectModeBeginEnd, ctx->BeginEnd, numEntries * sizeof(_glapi_proc));
 
-void GLAPIENTRY
-_es_VertexAttrib4f(GLuint index, GLfloat x, GLfloat y, GLfloat z, GLfloat w)
-{
-   VertexAttrib4f_nopos(index, x, y, z, w);
-}
-
-
-void GLAPIENTRY
-_es_VertexAttrib1f(GLuint indx, GLfloat x)
-{
-   VertexAttrib4f_nopos(indx, x, 0.0f, 0.0f, 1.0f);
-}
-
-
-void GLAPIENTRY
-_es_VertexAttrib1fv(GLuint indx, const GLfloat* values)
-{
-   VertexAttrib4f_nopos(indx, values[0], 0.0f, 0.0f, 1.0f);
-}
-
-
-void GLAPIENTRY
-_es_VertexAttrib2f(GLuint indx, GLfloat x, GLfloat y)
-{
-   VertexAttrib4f_nopos(indx, x, y, 0.0f, 1.0f);
-}
-
-
-void GLAPIENTRY
-_es_VertexAttrib2fv(GLuint indx, const GLfloat* values)
-{
-   VertexAttrib4f_nopos(indx, values[0], values[1], 0.0f, 1.0f);
-}
-
-
-void GLAPIENTRY
-_es_VertexAttrib3f(GLuint indx, GLfloat x, GLfloat y, GLfloat z)
-{
-   VertexAttrib4f_nopos(indx, x, y, z, 1.0f);
-}
-
-
-void GLAPIENTRY
-_es_VertexAttrib3fv(GLuint indx, const GLfloat* values)
-{
-   VertexAttrib4f_nopos(indx, values[0], values[1], values[2], 1.0f);
-}
-
-
-void GLAPIENTRY
-_es_VertexAttrib4fv(GLuint indx, const GLfloat* values)
-{
-   VertexAttrib4f_nopos(indx, values[0], values[1], values[2], values[3]);
+#undef NAME
+#define NAME(x) _hw_select_##x
+   struct _glapi_table *tab = ctx->HWSelectModeBeginEnd;
+   #include "api_hw_select_init.h"
 }
