@@ -55,8 +55,11 @@ util_queue_kill_threads(struct util_queue *queue, unsigned keep_num_threads,
  */
 
 static once_flag atexit_once_flag = ONCE_FLAG_INIT;
-static struct list_head queue_list;
-static mtx_t exit_mutex = _MTX_INITIALIZER_NP;
+static struct list_head queue_list = {
+   .next = &queue_list,
+   .prev = &queue_list,
+};
+static mtx_t exit_mutex;
 
 static void
 atexit_handler(void)
@@ -74,7 +77,7 @@ atexit_handler(void)
 static void
 global_init(void)
 {
-   list_inithead(&queue_list);
+   mtx_init(&exit_mutex, mtx_plain);
    atexit(atexit_handler);
 }
 
